@@ -1,11 +1,23 @@
 /****************************************************************************
-** $Id: qjpegio.cpp,v 1.3 1997/10/09 15:42:35 warwick Exp $
+** $Id: qjpegio.cpp,v 1.6.2.3 1998/10/29 14:58:16 warwick Exp $
 **
 ** Implementation of JPEG QImage IOHandler
 **
 ** Created : 970521
 **
-** Copyright (C) 1997 by Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of Qt Free Edition, version 1.45.
+**
+** See the file LICENSE included in the distribution for the usage
+** and distribution terms, or http://www.troll.no/free-license.html.
+**
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
@@ -15,9 +27,12 @@
 extern "C" {
 #define XMD_H // Shut JPEGlib up.
 #include <jpeglib.h>
+#ifdef const
+#undef const // Remove crazy C hackery in jconfig.h
+#endif
 }
 #include <qimage.h>
-#include <qiodev.h>
+#include <qiodevice.h>
 
 struct my_error_mgr : public jpeg_error_mgr {
     jmp_buf setjmp_buffer;
@@ -60,16 +75,19 @@ public:
 
     static boolean fill_input_buffer(j_decompress_ptr cinfo)
     {
+	int num_read;
 	my_jpeg_source_mgr* src = (my_jpeg_source_mgr*)cinfo->src;
 	QIODevice* dev = src->iio->ioDevice();
 	src->next_input_byte = src->buffer;
-	src->bytes_in_buffer = dev->readBlock((char*)src->buffer, max_buf);
-	if ( src->bytes_in_buffer <= 0 ) {
+	num_read = dev->readBlock((char*)src->buffer, max_buf);
+	if ( num_read <= 0 ) {
 	    // Insert a fake EOI marker - as per jpeglib recommendation
 	    src->buffer[0] = (JOCTET) 0xFF;
 	    src->buffer[1] = (JOCTET) JPEG_EOI;
 	    src->bytes_in_buffer = 2;
 	}
+	else
+	    src->bytes_in_buffer = num_read;
 	return TRUE;
     }
 
@@ -287,12 +305,12 @@ void write_jpeg_image(QImageIO* iio)
 		    uchar* data = image.scanLine(cinfo.next_scanline);
 		    if ( image.bitOrder() == QImage::LittleEndian ) {
 			for (int i=0; i<w; i++) {
-			    bool bit = !!*(data + (i >> 3)) & (1 << (i & 7));
+			    bool bit = !!(*(data + (i >> 3)) & (1 << (i & 7)));
 			    row[i] = qRed(cmap[bit]);
 			}
 		    } else {
 			for (int i=0; i<w; i++) {
-			    bool bit = !!*(data + (i >> 3)) & (1 << (7 -(i & 7)));
+			    bool bit = !!(*(data + (i >> 3)) & (1 << (7 -(i & 7))));
 			    row[i] = qRed(cmap[bit]);
 			}
 		    }
@@ -300,19 +318,17 @@ void write_jpeg_image(QImageIO* iio)
 		    uchar* data = image.scanLine(cinfo.next_scanline);
 		    if ( image.bitOrder() == QImage::LittleEndian ) {
 			for (int i=0; i<w; i++) {
-			    bool bit = !!*(data + (i >> 3)) & (1 << (i & 7));
-			    *row = qRed(cmap[bit]);
-			    *row = qGreen(cmap[bit]);
-			    *row = qBlue(cmap[bit]);
-			    ++row;
+			    bool bit = !!(*(data + (i >> 3)) & (1 << (i & 7)));
+			    *row++ = qRed(cmap[bit]);
+			    *row++ = qGreen(cmap[bit]);
+			    *row++ = qBlue(cmap[bit]);
 			}
 		    } else {
 			for (int i=0; i<w; i++) {
-			    bool bit = !!*(data + (i >> 3)) & (1 << (7 -(i & 7)));
-			    *row = qRed(cmap[bit]);
-			    *row = qGreen(cmap[bit]);
-			    *row = qBlue(cmap[bit]);
-			    ++row;
+			    bool bit = !!(*(data + (i >> 3)) & (1 << (7 -(i & 7))));
+			    *row++ = qRed(cmap[bit]);
+			    *row++ = qGreen(cmap[bit]);
+			    *row++ = qBlue(cmap[bit]);
 			}
 		    }
 		}
