@@ -1,32 +1,35 @@
 /****************************************************************************
-** $Id: qtimer.h,v 2.1 1997/08/06 15:05:47 hanord Exp $
+** $Id: qtimer.h,v 2.4.2.2 1998/08/21 19:13:24 hanord Exp $
 **
 ** Definition of QTimer class
 **
 ** Created : 931111
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QTIMER_H
 #define QTIMER_H
 
+#ifndef QT_H
 #include "qobject.h"
+#endif // QT_H
 
 
-class QTimer : public QObject
+class Q_EXPORT QTimer : public QObject
 {
     Q_OBJECT
 public:
@@ -52,8 +55,10 @@ private:
     bool	single;
 
 private:	// Disabled copy constructor and operator=
-    QTimer( const QTimer & ) {}
-    QTimer &operator=( const QTimer & ) { return *this; }
+#if defined(Q_DISABLE_COPY)
+    QTimer( const QTimer & );
+    QTimer &operator=( const QTimer & );
+#endif
 };
 
 
