@@ -1,70 +1,70 @@
 /****************************************************************************
-** $Id: qasyncimageio.cpp,v 1.21 1997/09/26 17:37:56 warwick Exp $
+** $Id: qasyncimageio.cpp,v 1.38.2.6 1999/02/13 14:57:55 hanord Exp $
 **
 ** Implementation of asynchronous image/movie loading classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
-#include <qpainter.h>
-#include <qlist.h>
 #include "qasyncimageio.h"
-
+#include "qpainter.h"
+#include "qlist.h"
+#include "qt_gif.h"
 #include <stdlib.h>
 
+
 /*!
   \class QImageConsumer qasyncimageio.h
   \brief An abstraction used by QImageDecoder.
 
-  \internal
+  \ingroup images
 
   A QImageConsumer consumes information about changes to the
   QImage maintained by a QImageDecoder.  It represents the
-  decoder's view of the obhect which uses the image it produces.
-  Each method of QImageConsumer returns a bool which should be
-  TRUE if the decoder should continue decoding.
+  a view of the image which the decoder produces.
 
   \sa QImageDecoder
 */
 
 /*!
-  \fn bool QImageConsumer::changed(const QRect&)
+  \fn void QImageConsumer::changed(const QRect&)
 
   Called when the given area of the image has changed.
 */
 
 /*!
-  \fn bool QImageConsumer::end()
+  \fn void QImageConsumer::end()
 
   Called when all data of all frames has been decoded and revealed
   as changed().
 */
 
 /*!
-  \fn bool QImageConsumer::frameDone()
+  \fn void QImageConsumer::frameDone()
 
   Called when a frame of an animated image has ended and been revealed
-  as changed().  If the consumer returns TRUE, the decoder will not make
+  as changed().  The decoder will not make
   any further changes to the image until the next call to
-  QImageFormatDecoder::decode().
+  QImageFormat::decode().
 */
 
 /*!
-  \fn bool QImageConsumer::setLooping(int n)
+  \fn void QImageConsumer::setLooping(int n)
 
   Called to indicate that the sequence of frames in the image
   should be repeated \a n times, including the sequence during
@@ -85,7 +85,7 @@
 */
 
 /*!
-  \fn bool QImageConsumer::setFramePeriod(int milliseconds)
+  \fn void QImageConsumer::setFramePeriod(int milliseconds)
 
   Notes that the frame about to be decoded should not be displayed until
   the given number of \a milliseconds after the time that this function
@@ -96,7 +96,7 @@
 */
 
 /*!
-  \fn bool QImageConsumer::setSize(int, int)
+  \fn void QImageConsumer::setSize(int, int)
 
   This function is called as soon as the size of the image has
   been determined.
@@ -107,26 +107,152 @@
   \class QImageDecoder qasyncimageio.h
   \brief Incremental image decoder for all supported image formats.
 
-  \internal
+  \ingroup images
 
   New formats are installed by creating objects of class
-  QImageFormatDecoderFactory.
+  QImageFormatType, and the QMovie class can be used for using
+  all installed incremental image formats; QImageDecoder is only
+  useful for creating new ways of feeding data to an QImageConsumer.
 */
 
 static const int max_header = 32;
 
+
+
+
+
+// See qt_gif.h for important information regarding this option
+#if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
+class Q_EXPORT QGIFFormat : public QImageFormat {
+public:
+    QGIFFormat();
+    virtual ~QGIFFormat();
+
+    int decode(QImage& img, QImageConsumer* consumer,
+	    const uchar* buffer, int length);
+
+private:
+    void fillRect(QImage&, int x, int y, int w, int h, uchar col);
+
+    // GIF specific stuff
+    QRgb* globalcmap;
+    QImage backingstore;
+    unsigned char hold[16];
+    bool gif89;
+    int count;
+    int ccount;
+    int expectcount;
+    enum State {
+	Header,
+	LogicalScreenDescriptor,
+	GlobalColorMap,
+	LocalColorMap,
+	Introducer,
+	ImageDescriptor,
+	TableImageLZWSize,
+	ImageDataBlockSize,
+	ImageDataBlock,
+	ExtensionLabel,
+	GraphicControlExtension,
+	ApplicationExtension,
+	NetscapeExtensionBlockSize,
+	NetscapeExtensionBlock,
+	SkipBlockSize,
+	SkipBlock,
+	Done,
+	Error
+    } state;
+    int gncols;
+    int ncols;
+    int lzwsize;
+    bool lcmap;
+    int swidth, sheight;
+    int left, top, right, bottom;
+    enum Disposal { NoDisposal, DoNotChange, RestoreBackground, RestoreImage };
+    Disposal disposal;
+    bool disposed;
+    int trans;
+    bool preserve_trans;
+    bool gcmap;
+    int bgcol;
+    int interlace;
+    int accum;
+    int bitcount;
+
+    enum { max_lzw_bits=12 }; // (poor-compiler's static const int)
+
+    int code_size, clear_code, end_code, max_code_size, max_code;
+    int firstcode, oldcode, incode;
+    short table[2][1<< max_lzw_bits];
+    short stack[(1<<(max_lzw_bits))*2];
+    short *sp;
+    bool needfirst;
+    int x, y;
+    int frame;
+    bool out_of_bounds;
+    bool digress;
+    void nextY(QImage& img, QImageConsumer* consumer);
+    void disposePrevious( QImage& img, QImageConsumer* consumer );
+};
+
+class Q_EXPORT QGIFFormatType : public QImageFormatType
+{
+    QImageFormat* decoderFor(const uchar* buffer, int length);
+    const char* formatName() const;
+};
+
+#endif
+
+
 struct QImageDecoderPrivate {
     QImageDecoderPrivate()
     {
 	count = 0;
     }
 
-    static QList<QImageFormatDecoderFactory> factories;
+    static void cleanup();
+
+    static void ensureFactories()
+    {
+	if ( !factories ) {
+	    factories = new QListT<QImageFormatType>;
+// See qt_gif.h for important information regarding this option
+#if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
+	    gif_decoder_factory = new QGIFFormatType;
+#endif
+	    qAddPostRoutine( cleanup );
+	}
+    }
+
+    static QList<QImageFormatType> * factories;
+
+// See qt_gif.h for important information regarding this option
+#if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
+    static QGIFFormatType * gif_decoder_factory;
+#endif
 
     uchar header[max_header];
     int count;
 };
-QList<QImageFormatDecoderFactory> QImageDecoderPrivate::factories;
+
+QList<QImageFormatType> * QImageDecoderPrivate::factories = 0;
+// See qt_gif.h for important information regarding this option
+#if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
+QGIFFormatType * QImageDecoderPrivate::gif_decoder_factory = 0;
+#endif
+
+
+void QImageDecoderPrivate::cleanup()
+{
+    delete factories;
+    factories = 0;
+// See qt_gif.h for important information regarding this option
+#if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
+    delete gif_decoder_factory;
+    gif_decoder_factory = 0;
+#endif
+}
+
 
 /*!
   Constructs a QImageDecoder which will send change information to
@@ -175,10 +301,12 @@ int QImageDecoder::decode(const uchar* buffer, int length)
 	while (consumed < length && d->count < max_header) {
 	    d->header[d->count++] = buffer[consumed++];
 	}
+	
+	QImageDecoderPrivate::ensureFactories();
 
-	for (QImageFormatDecoderFactory* f = QImageDecoderPrivate::factories.first();
+	for (QImageFormatType* f = QImageDecoderPrivate::factories->first();
 	    f && !actual_decoder;
-	    f = QImageDecoderPrivate::factories.next())
+	    f = QImageDecoderPrivate::factories->next())
 	{
 	    actual_decoder = f->decoderFor(d->header, d->count);
 	}
@@ -213,12 +341,14 @@ int QImageDecoder::decode(const uchar* buffer, int length)
 */
 const char* QImageDecoder::formatName(const uchar* buffer, int length)
 {
+    QImageDecoderPrivate::ensureFactories();
+
     const char* name = 0;
-    for (QImageFormatDecoderFactory* f = QImageDecoderPrivate::factories.first();
+    for (QImageFormatType* f = QImageDecoderPrivate::factories->first();
 	f && !name;
-	f = QImageDecoderPrivate::factories.next())
+	f = QImageDecoderPrivate::factories->next())
     {
-	QImageFormatDecoder *decoder = f->decoderFor(buffer, length);
+	QImageFormat *decoder = f->decoderFor(buffer, length);
 	if (decoder) {
 	    name = f->formatName();
 	    delete decoder;
@@ -232,10 +362,13 @@ const char* QImageDecoder::formatName(const uchar* buffer, int length)
 */
 QStrList QImageDecoder::inputFormats()
 {
+    QImageDecoderPrivate::ensureFactories();
+
     QStrList result;
 
-    for (QImageFormatDecoderFactory* f = QImageDecoderPrivate::factories.first();
-	f; f = QImageDecoderPrivate::factories.next())
+    for (QImageFormatType* f = QImageDecoderPrivate::factories->first();
+	 f;
+	 f = QImageDecoderPrivate::factories->next())
     {
 	if ( !result.contains(  f->formatName() ) ) {
 	    result.inSort(  f->formatName() );
@@ -246,28 +379,44 @@ QStrList QImageDecoder::inputFormats()
 }
 
 /*!
-  Registers a new QImageFormatDecoderFactory.  This is not needed in
+  Registers a new QImageFormatType.  This is not needed in
   application code as factories call this themselves.
 */
-void QImageDecoder::registerDecoderFactory(QImageFormatDecoderFactory* f)
+void QImageDecoder::registerDecoderFactory(QImageFormatType* f)
 {
-    QImageDecoderPrivate::factories.insert(0,f);
+    QImageDecoderPrivate::ensureFactories();
+
+    QImageDecoderPrivate::factories->insert(0,f);
 }
 
 /*!
-  Unregisters a new QImageFormatDecoderFactory.  This is not needed in
+  Unregisters a new QImageFormatType.  This is not needed in
   application code as factories call this themselves.
 */
-void QImageDecoder::unregisterDecoderFactory(QImageFormatDecoderFactory* f)
+void QImageDecoder::unregisterDecoderFactory(QImageFormatType* f)
 {
-    QImageDecoderPrivate::factories.remove(f);
+    if ( !QImageDecoderPrivate::factories )
+	return;
+
+    QImageDecoderPrivate::factories->remove(f);
 }
 
 /*!
-  \class QImageFormatDecoder qasyncimageio.h
+  \class QImageFormat qasyncimageio.h
   \brief Incremental image decoder for a specific image format.
 
-  \internal
+  \ingroup images
+
+  By making a derived classes of QImageFormatType which in turn
+  creates objects that are a subclass of QImageFormat, you can add
+  support for more incremental image formats, allowing such formats to
+  be sources for a QMovie, or for the first frame of the image stream
+  to be loaded as a QImage or QPixmap.
+
+  Your new subclass must override the decode() function in order to
+  process your new format.
+
+  New QImageFormat objects are generated by new QImageFormatType factories.
 */
 
 /*!
@@ -276,33 +425,57 @@ void QImageDecoder::unregisterDecoderFactory(QImageFormatDecoderFactory* f)
   \internal
   More importantly, destructs derived classes.
 */
-QImageFormatDecoder::~QImageFormatDecoder()
+QImageFormat::~QImageFormat()
 {
 }
 
 /*!
-  \fn int QImageFormatDecoder::decode(QImage& img, QImageConsumer* consumer,
+  \fn int QImageFormat::decode(QImage& img, QImageConsumer* consumer,
 	    const uchar* buffer, int length)
 
-  Image decoders for specific image formats must override this method.
-  It should decode some or all of the bytes in the given buffer into the
-  given image, calling the given consumer as the decoding proceeds.
-  The consumer may be 0.
+  New subclasses must override this method.
+
+  It should decode some or all of the bytes from \a buffer into
+  \a img, calling the methods of \a consumer as the decoding proceeds to
+  inform that consumer of changes to the image.
+  The consumer may be 0, in which case the function should just process
+  the data into \a img without telling any consumer about the changes.
+  Note that the decoder must store enough state
+  to be able to continue in subsequent calls to this method - this is
+  the essence of the incremental image loading.
+
+  The function should return without processing all the data if it
+  reaches the end of a frame in the input.
+
+  The function must return the number of bytes it has processed.
 */
 
 /*!
-  \class QImageFormatDecoderFactory qasyncimageio.h
-  \brief Factory that makes QImageFormatDecoder objects.
+  \class QImageFormatType qasyncimageio.h
+  \brief Factory that makes QImageFormat objects.
 
-  \internal
+  \ingroup images
+
+  While the QImageIO class allows for \e complete loading of images,
+  QImageFormatType allows for \e incremental loading of images.
 
   New image file formats are installed by creating objects of derived
-  classes of QImageFormatDecoderFactory.  They must implement decoderFor()
+  classes of QImageFormatType.  They must implement decoderFor()
   and formatName().
+
+  QImageFormatType is a very simple class.  Its only task is to
+  recognize image data in some format and make a new object, subclassed
+  from QImageFormat, which can decode that format.
+
+  The factories for formats built into Qt
+  are automatically defined before any other factory is initialized.
+  If two factories would recognize an image format, the factory created
+  last will override the earlier one, thus you can override current
+  and future built-in formats.
 */
 
 /*!
-  \fn virtual QImageFormatDecoder* QImageFormatDecoderFactory::decoderFor(const
+  \fn virtual QImageFormat* QImageFormatType::decoderFor(const
 	    uchar* buffer, int length)
 
   Returns a decoder for decoding an image which starts with the give bytes.
@@ -313,7 +486,7 @@ QImageFormatDecoder::~QImageFormatDecoder()
 */
 
 /*!
-  \fn virtual const char* QImageFormatDecoderFactory::formatName() const
+  \fn virtual const char* QImageFormatType::formatName() const
 
   Returns the name of the format supported by decoders from this factory.
   The string is statically allocated.
@@ -322,7 +495,7 @@ QImageFormatDecoder::~QImageFormatDecoder()
 /*!
   Creates a factory.  It automatically registers itself with QImageDecoder.
 */
-QImageFormatDecoderFactory::QImageFormatDecoderFactory()
+QImageFormatType::QImageFormatType()
 {
     QImageDecoder::registerDecoderFactory(this);
 }
@@ -330,25 +503,42 @@ QImageFormatDecoderFactory::QImageFormatDecoderFactory()
 /*!
   Destroys a factory.  It automatically unregisters itself from QImageDecoder.
 */
-QImageFormatDecoderFactory::~QImageFormatDecoderFactory()
+QImageFormatType::~QImageFormatType()
 {
     QImageDecoder::unregisterDecoderFactory(this);
 }
 
-/*!
-  \class QGIFDecoder qasyncimageio.h
+
+bool qt_builtin_gif_reader()
+{
+#if defined(QT_BUILTIN_GIF_READER)
+    return QT_BUILTIN_GIF_READER == 1;
+#else
+    return 0;
+#endif
+}
+
+// See qt_gif.h for important information regarding this option
+#if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
+
+/* -- NOTDOC
+  \class QGIFFormat qasyncimageio.h
   \brief Incremental image decoder for GIF image format.
 
-  \internal
+  \ingroup images
+
+  This subclass of QImageFormat decodes GIF format images,
+  including animated GIFs.  Internally in
 */
 
 /*!
-  Constructs a QGIFDecoder.
+  Constructs a QGIFFormat.
 */
-QGIFDecoder::QGIFDecoder()
+QGIFFormat::QGIFFormat()
 {
-    globalcmap_hold = 0;
+    globalcmap = 0;
     disposal = NoDisposal;
+    out_of_bounds = FALSE;
     disposed = TRUE;
     frame = -1;
     state = Header;
@@ -357,24 +547,27 @@ QGIFDecoder::QGIFDecoder()
 }
 
 /*!
-  Destructs a QGIFDecoder.
+  Destructs a QGIFFormat.
 */
-QGIFDecoder::~QGIFDecoder()
+QGIFFormat::~QGIFFormat()
 {
-    delete globalcmap_hold;
+    delete[] globalcmap;
 }
 
 
-class QGIFDecoderFactory : public QImageFormatDecoderFactory {
-    QImageFormatDecoder* decoderFor(const uchar* buffer, int length);
-    const char* formatName() const;
-};
+/* -- NOTDOC
+  \class QGIFFormatType qasyncimageio.h
+  \brief Incremental image decoder for GIF image format.
 
-// this variable must exist to read gifs, and if it's static sgi cc 7
-// complains... so give it an obscure name and make it non-static
-QGIFDecoderFactory qt_internal_gif_decoder_factory;
+  \ingroup images
 
-QImageFormatDecoder* QGIFDecoderFactory::decoderFor(
+  This subclass of QImageFormatType recognizes GIF
+  format images, creating a QGIFFormat when required.  An instance
+  of this class is created automatically before any other factories,
+  so you should have no need for such objects.
+*/
+
+QImageFormat* QGIFFormatType::decoderFor(
     const uchar* buffer, int length)
 {
     if (length < 6) return 0;
@@ -384,22 +577,30 @@ QImageFormatDecoder* QGIFDecoderFactory::decoderFor(
      && buffer[3]=='8'
      && (buffer[4]=='9' || buffer[4]=='7')
      && buffer[5]=='a')
-        return new QGIFDecoder;
+        return new QGIFFormat;
     return 0;
 }
 
-const char* QGIFDecoderFactory::formatName() const
+const char* QGIFFormatType::formatName() const
 {
     return "GIF";
 }
 
 
-void QGIFDecoder::disposePrevious( QImage& img, QImageConsumer* consumer )
+void QGIFFormat::disposePrevious( QImage& img, QImageConsumer* consumer )
 {
+    if ( out_of_bounds ) // flush anything that survived
+	consumer->changed(QRect(0,0,swidth,sheight));
+
     // Handle disposal of previous image before processing next one
 
     if ( disposed ) return;
 
+    int l = QMIN(swidth-1,left);
+    int r = QMIN(swidth-1,right);
+    int t = QMIN(sheight-1,top);
+    int b = QMIN(sheight-1,bottom);
+
     switch (disposal) {
       case NoDisposal:
 	break;
@@ -409,34 +610,27 @@ void QGIFDecoder::disposePrevious( QImage& img, QImageConsumer* consumer )
 	preserve_trans = FALSE;
 	if (trans>=0) {
 	    // Easy:  we use the transparent colour
-	    fillRect(img, left, top, right-left+1, bottom-top+1, trans);
+	    fillRect(img, l, t, r-l+1, b-t+1, trans);
 	} else if (bgcol>=0) {
 	    // Easy:  we use the bgcol given
-	    fillRect(img, left, top, right-left+1, bottom-top+1, bgcol);
+	    fillRect(img, l, t, r-l+1, b-t+1, bgcol);
 	} else {
 	    // Impossible:  We don't know of a bgcol - use pixel 0
 	    uchar** line = img.jumpTable();
-	    fillRect(img, left, top, right-left+1, bottom-top+1, line[0][0]);
-	}
-	if (consumer) {
-	    bool d = !consumer->changed(QRect(left, top,
-					      right-left+1, bottom-top+1));
-	    digress = digress || d;
+	    fillRect(img, l, t, r-l+1, b-t+1, line[0][0]);
 	}
+	if (consumer)
+	    consumer->changed(QRect(l, t, r-l+1, b-t+1));
 	break;
       case RestoreImage: {
 	uchar** line = img.jumpTable();
 	preserve_trans = FALSE;
-	for (int ln=top; ln<=bottom; ln++) {
-	    memcpy(line[ln]+left,
-		backingstore.scanLine(ln-top),
-		right-left+1);
-	}
-	if (consumer) {
-	    bool d = !consumer->changed(QRect(left, top,
-					      right-left+1, bottom-top+1));
-	    digress = digress || d;
+	for (int ln=t; ln<=b; ln++) {
+	    memcpy(line[ln]+l,
+		backingstore.scanLine(ln-t),
+		r-l+1);
 	}
+	consumer->changed(QRect(l, t, r-l+1, b-t+1));
       }
     }
     disposal = NoDisposal; // Until an extension says otherwise.
@@ -449,7 +643,7 @@ void QGIFDecoder::disposePrevious( QImage& img, QImageConsumer* consumer )
 
   Returns the number of bytes consumed.
 */
-int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
+int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 	const uchar* buffer, int length)
 {
     // We are required to state that
@@ -487,10 +681,6 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 		bgcol=(gcmap && hold[5]) ? hold[5] : -1;
 		//aspect=hold[6] ? double(hold[6]+15)/64.0 : 1.0;
 
-		img.create(swidth, sheight, 8, gcmap ? gncols : 256);
-		line = img.jumpTable();
-		if (consumer) digress = !consumer->setSize(swidth, sheight);
-
 		trans = -1;
 		preserve_trans = FALSE;
 		count=0;
@@ -498,6 +688,7 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 		if (gcmap) {
 		    ccount=0;
 		    state=GlobalColorMap;
+		    globalcmap = new QRgb[gncols];
 		} else {
 		    state=ImageDescriptor;
 		}
@@ -506,9 +697,13 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 	  case GlobalColorMap: case LocalColorMap:
 	    hold[count++]=ch;
 	    if (count==3) {
-		img.setColor(ccount,
-		    (ccount==trans ? 0 : 0xff000000)
-		    | qRgb(hold[0], hold[1], hold[2]));
+		QRgb rgb = (ccount==trans ? 0 : 0xff000000)
+		    | qRgb(hold[0], hold[1], hold[2]);
+		if ( state == LocalColorMap ) {
+		    img.setColor(ccount, rgb);
+		} else {
+		    globalcmap[ccount] = rgb;
+		}
 		if (++ccount >= ncols) {
 		    if ( state == LocalColorMap )
 			state=TableImageLZWSize;
@@ -528,7 +723,11 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 		state=ExtensionLabel;
 		break;
 	      case ';':
-		if (consumer) digress = !consumer->end();
+		if (consumer) {
+		    if ( out_of_bounds ) // flush anything that survived
+			consumer->changed(QRect(0,0,swidth,sheight));
+		    consumer->end();
+		}
 		state=Done;
 		break;
 	      default:
@@ -540,15 +739,47 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 	  case ImageDescriptor:
 	    hold[count++]=ch;
 	    if (count==10) {
-		disposePrevious( img, consumer );
-		disposed = FALSE;
-		left=LM(hold[1], hold[2]);
-		top=LM(hold[3], hold[4]);
+		int newleft=LM(hold[1], hold[2]);
+		int newtop=LM(hold[3], hold[4]);
 		int width=LM(hold[5], hold[6]);
 		int height=LM(hold[7], hold[8]);
+
+		if ( swidth <= 0 )
+		    swidth = newleft + width;
+		if ( sheight <= 0 )
+		    sheight = newtop + height;
+
+		if (img.isNull()) {
+		    img.create(swidth, sheight, 8, gcmap ? gncols : 256);
+		    if (consumer) consumer->setSize(swidth, sheight);
+		}
+		img.setAlphaBuffer(trans >= 0);
+		line = img.jumpTable();
+
+		disposePrevious( img, consumer );
+		disposed = FALSE;
+
+		left = newleft;
+		top = newtop;
+
+		// Sanity check frame size - must fit on "screen".
+		if (left >= swidth) left=swidth-1;
+		if (top >= sheight) top=sheight-1;
+		if (left+width >= swidth) {
+		    if ( width <= swidth )
+			left=swidth-width;
+		    else
+			width=swidth-left;
+		}
+		if (top+height >= sheight) {
+		    if ( height <= sheight )
+			top=sheight-height;
+		    else
+			height=sheight-top;
+		}
+
 		right=left+width-1;
 		bottom=top+height-1;
-		bool hadlcmap = lcmap;
 		lcmap=!!(hold[9]&0x80);
 		interlace=!!(hold[9]&0x40);
 		//bool lcmsortflag=!!(hold[9]&0x20);
@@ -565,27 +796,34 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 			// Not full-size image - erase with bg or transparent
 			if ( bgcol>=0 ) {
 			    fillRect(img, 0, 0, swidth, sheight, bgcol);
-			    if (consumer) digress = !consumer->changed(QRect(0,0,swidth,sheight));
+			    if (consumer) consumer->changed(QRect(0,0,swidth,sheight));
 			} else if ( trans > 0 ) {
 			    fillRect(img, 0, 0, swidth, sheight, trans);
-			    if (consumer) digress = !consumer->changed(QRect(0,0,swidth,sheight));
+			    if (consumer) consumer->changed(QRect(0,0,swidth,sheight));
 			}
 		    }
 		}
 
 		if ( disposal == RestoreImage ) {
-		    if (backingstore.width() < width
-			|| backingstore.height() < height) {
+		    int l = QMIN(swidth-1,left);
+		    int r = QMIN(swidth-1,right);
+		    int t = QMIN(sheight-1,top);
+		    int b = QMIN(sheight-1,bottom);
+		    int w = r-l+1;
+		    int h = b-t+1;
+
+		    if (backingstore.width() < w
+			|| backingstore.height() < h) {
 			// We just use the backing store as a byte array
 			backingstore.create( QMAX(backingstore.width(),
-						  width),
+						  w),
 					     QMAX(backingstore.height(),
-						  height),
+						  h),
 					     8,1);
 		    }
-		    for (int ln=0; ln<height; ln++) {
+		    for (int ln=0; ln<h; ln++) {
 			memcpy(backingstore.scanLine(ln),
-			       line[top+ln]+left, width);
+			       line[t+ln]+l, w);
 		    }
 		}
 
@@ -593,17 +831,9 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 		if (lcmap) {
 		    ccount=0;
 		    state=LocalColorMap;
-		    if (gcmap) {
-			if (!globalcmap_hold) {
-			    globalcmap_hold = new QRgb[256];
-			    CHECK_PTR(globalcmap_hold);
-			}
-			memcpy(globalcmap_hold, img.colorTable(),
-			    ncols * sizeof(QRgb));
-		    }
 		} else {
-		    if (hadlcmap && gcmap) {
-			memcpy(img.colorTable(), globalcmap_hold,
+		    if (gcmap) {
+			memcpy(img.colorTable(), globalcmap,
 			    ncols * sizeof(QRgb));
 		    }
 		    state=TableImageLZWSize;
@@ -614,6 +844,7 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 		bitcount = 0;
 		sp = stack;
 		needfirst = FALSE;
+		out_of_bounds = FALSE;
 	    }
 	    break;
 	  case TableImageLZWSize: {
@@ -643,7 +874,10 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 	    if (expectcount) {
 		state=ImageDataBlock;
 	    } else {
-		if (consumer) digress = !consumer->frameDone();
+		if (consumer) {
+		    consumer->frameDone();
+		    digress = TRUE;
+		}
 
 		state=Introducer;
 	    }
@@ -678,12 +912,15 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 		} else {
 		    if (needfirst) {
 			firstcode=oldcode=code;
-			if (!(preserve_trans && firstcode==trans))
+			if (!out_of_bounds && !(preserve_trans && firstcode==trans))
 			    line[y][x] = firstcode;
 			x++;
+			if (x>=swidth) out_of_bounds = TRUE;
 			needfirst=FALSE;
 			if (x>right) {
 			    x=left;
+			    if (out_of_bounds)
+				out_of_bounds = left>=swidth || y>=sheight;
 			    nextY(img,consumer);
 			}
 		    } else {
@@ -720,11 +957,14 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 			oldcode=incode;
 			while (sp>stack) {
 			    --sp;
-			    if (!(preserve_trans && *sp==trans))
+			    if (!out_of_bounds && !(preserve_trans && *sp==trans))
 				line[y][x] = *sp;
 			    x++;
+			    if (x>=swidth) out_of_bounds = TRUE;
 			    if (x>right) {
 				x=left;
+				if (out_of_bounds)
+				    out_of_bounds = left>=swidth || y>=sheight;
 				nextY(img,consumer);
 			    }
 			}
@@ -803,9 +1043,9 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 		if (newtrans >= ncols) {
 		    // Ignore invalid transparency.
 		    newtrans=-1;
-		    havetrans=FALSE;
 		}
-		if (trans >= 0) preserve_trans = TRUE;
+		if (newtrans >= 0 && frame>=0)
+		    preserve_trans = TRUE;
 		if (newtrans != trans) {
 		    // Unset old transparency
 		    if (trans >= 0) {
@@ -825,13 +1065,10 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 		    }
 		    trans = newtrans;
 		    if (trans >= 0) {
-			if (globalcmap_hold) {
-			    globalcmap_hold[trans]&=0x00ffffff;
-			}
-			img.setColor(trans, 0x00ffffff&img.color(trans));
+			if (globalcmap)
+			    globalcmap[trans]&=0x00ffffff;
 		    }
 		}
-		img.setAlphaBuffer(havetrans);
 		if (consumer) consumer->setFramePeriod(delay*10);
 		count=0;
 		state=SkipBlockSize;
@@ -859,7 +1096,7 @@ int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
     return initial-length;
 }
 
-void QGIFDecoder::fillRect(QImage& img, int col, int row, int w, int h, uchar color)
+void QGIFFormat::fillRect(QImage& img, int col, int row, int w, int h, uchar color)
 {
     if (w>0) {
 	uchar** line = img.jumpTable() + row;
@@ -869,14 +1106,14 @@ void QGIFDecoder::fillRect(QImage& img, int col, int row, int w, int h, uchar co
     }
 }
 
-void QGIFDecoder::nextY(QImage& img, QImageConsumer* consumer)
+void QGIFFormat::nextY(QImage& img, QImageConsumer* consumer)
 {
     int my;
     switch (interlace) {
       case 0:
 	// Non-interlaced
-	if (consumer) digress =
-	    !consumer->changed(QRect(left, y, right-left+1, 1));
+	if (consumer && !out_of_bounds)
+	    consumer->changed(QRect(left, y, right-left+1, 1));
 	y++;
 	break;
       case 1:
@@ -885,8 +1122,8 @@ void QGIFDecoder::nextY(QImage& img, QImageConsumer* consumer)
 	    my = QMIN(7, bottom-y);
 	    for (i=1; i<=my; i++)
 	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
-	    if (consumer) digress =
-	        !consumer->changed(QRect(left, y, right-left+1, my+1));
+	    if (consumer && !out_of_bounds)
+	        consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=8;
 	    if (y>bottom) { interlace++; y=4; }
 	} break;
@@ -896,8 +1133,8 @@ void QGIFDecoder::nextY(QImage& img, QImageConsumer* consumer)
 	    my = QMIN(3, bottom-y);
 	    for (i=1; i<=my; i++)
 	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
-    	    if (consumer) digress =
-	        !consumer->changed(QRect(left, y, right-left+1, my+1));
+    	    if (consumer && !out_of_bounds)
+	        consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=8;
 	    if (y>bottom) { interlace++; y=2; }
 	} break;
@@ -906,17 +1143,19 @@ void QGIFDecoder::nextY(QImage& img, QImageConsumer* consumer)
 	    my = QMIN(1, bottom-y);
 	    for (int i=1; i<=my; i++)
 	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
-	    if (consumer) digress =
-	        !consumer->changed(QRect(left, y, right-left+1, my+1));
+	    if (consumer && !out_of_bounds)
+	        consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=4;
 	    if (y>bottom) { interlace++; y=1; }
 	} break;
       case 4:
-	if (consumer) digress =
-	    !consumer->changed(QRect(left, y, right-left+1, 1));
+	if (consumer && !out_of_bounds)
+	    consumer->changed(QRect(left, y, right-left+1, 1));
 	y+=2;
     }
 
     // Consume bogus extra lines
-    if (y > bottom) y=bottom;
+    if (y >= sheight) out_of_bounds=TRUE; //y=bottom;
 }
+
+#endif // QT_BUILTIN_GIF_READER
