@@ -1,13 +1,13 @@
 /****************************************************************************
-** $Id: qdnd_x11.cpp,v 2.41 1998/07/03 00:09:32 hanord Exp $
+** $Id: qdnd_x11.cpp,v 2.41.2.14 1998/12/17 10:52:49 paul Exp $
 **
-** XDND implementation for Qt.  See http://www.cco.caltech.edu/~jafl/xdnd2/
+** XDND implementation for Qt.  See http://www.cco.caltech.edu/~jafl/xdnd/
 **
 ** Created : 980320
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/free-license.html.
@@ -15,7 +15,7 @@
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
+** Please see http://www.troll.no/pricing.html for information about
 ** Qt Professional Edition, which is this same library but with a
 ** license which allows creation of commercial/proprietary software.
 **
@@ -73,6 +73,8 @@ Atom qt_xdnd_status;
 Atom qt_xdnd_leave;
 Atom qt_xdnd_drop;
 Atom qt_xdnd_finished;
+// other atoms
+Atom qt_xdnd_action_copy;
 
 // end of copied stuff
 
@@ -118,6 +120,33 @@ QIntDict<QByteArray> * qt_xdnd_target_data = 0;
 // first drag object, or 0
 QDragObject * qt_xdnd_source_object = 0;
 
+class QShapedPixmapWidget : public QWidget {
+    QPixmap pixmap;
+public:
+    QShapedPixmapWidget() :
+	QWidget(0,0,WStyle_Customize | WStyle_Tool | WStyle_NoBorder)
+    {
+    }
+
+    void setPixmap(QPixmap pm)
+    {
+	pixmap = pm;
+	if ( pixmap.mask() ) {
+	    setMask( *pixmap.mask() );
+	} else {
+	    clearMask();
+	}
+	resize(pm.width(),pm.height());
+    }
+
+    void paintEvent(QPaintEvent*)
+    {
+	bitBlt(this,0,0,&pixmap);
+    }
+};
+
+QShapedPixmapWidget * qt_xdnd_deco = 0;
+
 const char* qt_xdnd_atom_to_str( Atom a )
 {
     if ( !a ) return 0;
@@ -178,6 +207,9 @@ void qt_xdnd_setup() {
 
     qt_x11_intern_atom( "XdndAware", &qt_xdnd_aware );
 
+
+    qt_x11_intern_atom( "XdndActionCopy", &qt_xdnd_action_copy );
+
     qt_x11_intern_atom( "QT_SELECTION", &qt_selection_property );
     qt_x11_intern_atom( "INCR", &qt_incr_atom );
 
@@ -234,7 +266,7 @@ void qt_handle_xdnd_enter( QWidget *, const XEvent * xe )
 
     const long *l = xe->xclient.data.l;
     int version = (int)(((unsigned long)(l[1])) >> 24);
-    
+
     if ( version > 2 )
 	return;
 
@@ -257,12 +289,15 @@ void qt_handle_xdnd_enter( QWidget *, const XEvent * xe )
 void qt_handle_xdnd_position( QWidget *w, const XEvent * xe )
 {
     const unsigned long *l = (const unsigned long *)xe->xclient.data.l;
-    
+
     QPoint p( (l[2] & 0xffff0000) >> 16, l[2] & 0x0000ffff );
     QWidget * c = find_child( w, p );
 
+    if ( !c || !c->acceptDrops() && c->isDesktop() )
+	return;
+
     if ( l[0] != qt_xdnd_dragsource_xid ) {
-	//	debug( "xdnd drag position from unexpected source (%08lx not %08lx)",
+	//debug( "xdnd drag position from unexpected source (%08lx not %08lx)",
 	//      l[0], qt_xdnd_dragsource_xid );
 	return;
     }
@@ -304,6 +339,9 @@ void qt_handle_xdnd_position( QWidget *w, const XEvent * xe )
     if ( !c->acceptDrops() ) {
 	qt_xdnd_current_widget = 0;
 	answerRect = QRect( p, QSize( 1, 1 ) );
+    } else if ( l[4] != qt_xdnd_action_copy ) {
+	response.data.l[0] = 0;
+	answerRect = QRect( p, QSize( 1, 1 ) );
     } else {
 	qt_xdnd_current_widget = c;
 	qt_xdnd_current_position = p;
@@ -335,13 +373,21 @@ void qt_handle_xdnd_position( QWidget *w, const XEvent * xe )
 
     response.data.l[2] = (answerRect.x() << 16) + answerRect.y();
     response.data.l[3] = (answerRect.width() << 16) + answerRect.height();
+    response.data.l[4] = qt_xdnd_action_copy;
 
     QWidget * source = QWidget::find( qt_xdnd_dragsource_xid );
+
+    int emask = NoEventMask;
+    if ( source && source->isDesktop() && !source->acceptDrops() ) {
+	emask = EnterWindowMask;
+	source = 0;
+    }
+
     if ( source )
 	qt_handle_xdnd_status( source, (const XEvent *)&response );
     else
-	XSendEvent( w->x11Display(), qt_xdnd_dragsource_xid, FALSE,
-		    NoEventMask, (XEvent*)&response );
+	XSendEvent( qt_xdisplay(), qt_xdnd_dragsource_xid, FALSE,
+		    emask, (XEvent*)&response );
 }
 
 
@@ -353,7 +399,8 @@ void qt_handle_xdnd_status( QWidget * w, const XEvent * xe )
 
     QPoint p( (l[2] & 0xffff0000) >> 16, l[2] & 0x0000ffff );
     QSize s( (l[3] & 0xffff0000) >> 16, l[3] & 0x0000ffff );
-    if ( s.width() < 4096 && s.height() < 4097 &&
+    if ( (int)(l[1] & 2) == 0 &&
+	 s.width() < 4096 && s.height() < 4097 &&
 	 s.width() > 0 && s.height() > 0 )
 	qt_xdnd_source_sameanswer = QRect( p, s );
     else
@@ -366,7 +413,6 @@ void qt_handle_xdnd_leave( QWidget *w, const XEvent * xe )
     //debug( "xdnd leave" );
     if ( !qt_xdnd_current_widget ||
 	 w->topLevelWidget() != qt_xdnd_current_widget->topLevelWidget() ) {
-	qt_xdnd_dragsource_xid = 0;
 	return; // sanity
     }
 
@@ -406,11 +452,18 @@ static void qt_xdnd_send_leave()
     leave.data.l[4] = 0; // just null
 
     QWidget * w = QWidget::find( qt_xdnd_current_target );
+
+    int emask = NoEventMask;
+    if ( w && w->isDesktop() && !w->acceptDrops() ) {
+	emask = EnterWindowMask;
+	w = 0;
+    }
+
     if ( w )
 	qt_handle_xdnd_leave( w, (const XEvent *)&leave );
     else
-	XSendEvent( w->x11Display(), qt_xdnd_current_target, FALSE,
-		    NoEventMask, (XEvent*)&leave );
+	XSendEvent( qt_xdisplay(), qt_xdnd_current_target, FALSE,
+		    emask, (XEvent*)&leave );
     qt_xdnd_current_target = 0;
 }
 
@@ -452,14 +505,19 @@ void qt_handle_xdnd_finished( QWidget *, const XEvent * xe )
 
 static QCursor *noDropCursor = 0;
 
-#define noDropCursorWidth 18
-#define noDropCursorHeight 18
+#define noDropCursorWidth 20
+#define noDropCursorHeight 20
 static unsigned char noDropCutBits[] = {
- 0xc0,0x0f,0xfc,0xf0,0x3f,0xfc,0x78,0x78,0xfc,0x1c,0xe0,0xfc,0x3e,0xc0,0xfd,
- 0x76,0x80,0xfd,0xe7,0x80,0xff,0xc3,0x01,0xff,0x83,0x03,0xff,0x03,0x07,0xff,
- 0x03,0x0e,0xff,0x07,0x9c,0xff,0x06,0xb8,0xfd,0x0e,0xf0,0xfd,0x1c,0xe0,0xfc,
- 0x78,0x70,0xfc,0xf0,0x3f,0xfc,0xc0,0x0f,0xfc};
+ 0x00,0x00,0x00,0x80,0x1f,0x00,0xe0,0x7f,0x00,0xf0,0xf0,0x00,0x38,0xc0,0x01,
+ 0x7c,0x80,0x03,0xec,0x00,0x03,0xce,0x01,0x07,0x86,0x03,0x06,0x06,0x07,0x06,
+ 0x06,0x0e,0x06,0x06,0x1c,0x06,0x0e,0x38,0x07,0x0c,0x70,0x03,0x1c,0xe0,0x03,
+ 0x38,0xc0,0x01,0xf0,0xe0,0x00,0xe0,0x7f,0x00,0x80,0x1f,0x00,0x00,0x00,0x00};
 
+static unsigned char noDropCutMask[] = {
+ 0x80,0x1f,0x00,0xe0,0x7f,0x00,0xf0,0xff,0x00,0xf8,0xff,0x01,0xfc,0xf0,0x03,
+ 0xfe,0xc0,0x07,0xfe,0x81,0x07,0xff,0x83,0x0f,0xcf,0x07,0x0f,0x8f,0x0f,0x0f,
+ 0x0f,0x1f,0x0f,0x0f,0x3e,0x0f,0x1f,0xfc,0x0f,0x1e,0xf8,0x07,0x3e,0xf0,0x07,
+ 0xfc,0xe0,0x03,0xf8,0xff,0x01,0xf0,0xff,0x00,0xe0,0x7f,0x00,0x80,0x1f,0x00};
 
 
 bool QDragManager::eventFilter( QObject * o, QEvent * e)
@@ -519,7 +577,8 @@ bool QDragManager::eventFilter( QObject * o, QEvent * e)
 	} else {
 	    if ( !noDropCursor ) {
 		QBitmap b( noDropCursorWidth, noDropCursorHeight, noDropCutBits, TRUE );
-		noDropCursor = new QCursor( b, b );
+		QBitmap m( noDropCursorWidth, noDropCursorHeight, noDropCutMask, TRUE );
+		noDropCursor = new QCursor( b, m );
 	    }
 	    QApplication::setOverrideCursor( *noDropCursor, restoreCursor );
 	    restoreCursor = TRUE;
@@ -549,8 +608,63 @@ void QDragManager::cancel()
     }
 
     qt_xdnd_source_object = 0;
+    delete qt_xdnd_deco;
+    qt_xdnd_deco = 0;
 }
 
+static
+Window findRealWindow( const QPoint & pos, Window w, int md )
+{
+    if ( qt_xdnd_deco && w == qt_xdnd_deco->winId() )
+	return 0;
+
+    if ( md ) {
+	XWindowAttributes attr;
+	XGetWindowAttributes( qt_xdisplay(), w, &attr );
+
+	if ( attr.map_state != IsUnmapped
+	    && QRect(attr.x,attr.y,attr.width,attr.height)
+		.contains(pos) )
+	{
+	    {
+		Atom   type = None;
+		int f;
+		unsigned long n, a;
+		unsigned char *data;
+
+		XGetWindowProperty( qt_xdisplay(), w, qt_wm_state, 0,
+		    0, False, AnyPropertyType, &type, &f,&n,&a,&data );
+
+		if ( data )
+		    XFree(data);
+
+		if ( type )
+		    return w;
+	    }
+
+	    Window r, p;
+	    Window* c;
+	    uint nc;
+	    if ( XQueryTree( qt_xdisplay(), w, &r, &p, &c, &nc ) ) {
+		r=0;
+		for (uint i=nc; !r && i--; ) {
+		    r = findRealWindow( pos-QPoint(attr.x,attr.y),
+					c[i], md-1 );
+		}
+		XFree((void*)c);
+		if ( r )
+		    return r;
+
+		// We didn't find a client window!  Just use the
+		// innermost window.
+	    }
+
+	    // No children!
+	    return w;
+	}
+    }
+    return 0;
+}
 
 void QDragManager::move( const QPoint & globalPos )
 {
@@ -560,6 +674,11 @@ void QDragManager::move( const QPoint & globalPos )
 	return;
     }
 
+    if ( qt_xdnd_deco ) {
+	qt_xdnd_deco->move(globalPos-qt_xdnd_source_object->pixmapHotSpot());
+	qt_xdnd_deco->raise();
+    }
+
     Window target = 0;
     int lx = 0, ly = 0;
     if ( !XTranslateCoordinates( qt_xdisplay(), qt_xrootwin(), qt_xrootwin(),
@@ -569,14 +688,26 @@ void QDragManager::move( const QPoint & globalPos )
 	return;
     }
 
-    if ( target != 0 )
+    if ( target == qt_xrootwin() ) {
+	// Ok.
+    } else if ( target ) {
 	target = qt_x11_findClientWindow( target, qt_wm_state, TRUE );
+	if ( qt_xdnd_deco && !target || target == qt_xdnd_deco->winId() ) {
+	    target = findRealWindow(globalPos,qt_xrootwin(),6);
+	}
+    }
 
     if ( target == 0 )
 	target = qt_xrootwin();
 
     QWidget * w = QWidget::find( (WId)target );
 
+    int emask = NoEventMask;
+    if ( w && w->isDesktop() && !w->acceptDrops() ) {
+	emask = EnterWindowMask;
+	w = 0;
+    }
+
     if ( target != qt_xdnd_current_target ) {
 	if ( qt_xdnd_current_target )
 	    qt_xdnd_send_leave();
@@ -602,12 +733,12 @@ void QDragManager::move( const QPoint & globalPos )
 	qt_xdnd_source_sameanswer = QRect( globalPos.x() - 2,
 					   globalPos.y() -2 , 5, 5 );
 
-	if ( w )
+	if ( w ) {
 	    qt_handle_xdnd_enter( w, (const XEvent *)&enter );
-	else
-	    XSendEvent( qt_xdisplay(), target, FALSE, NoEventMask,
+	} else {
+	    XSendEvent( qt_xdisplay(), target, FALSE, emask,
 			(XEvent*)&enter );
-
+	}
     }
 
     XClientMessageEvent move;
@@ -620,12 +751,12 @@ void QDragManager::move( const QPoint & globalPos )
     move.data.l[1] = 0; // flags
     move.data.l[2] = (globalPos.x() << 16) + globalPos.y();
     move.data.l[3] = qt_x_clipboardtime;
-    move.data.l[4] = 0;
+    move.data.l[4] = qt_xdnd_action_copy;
 
     if ( w )
 	qt_handle_xdnd_position( w, (const XEvent *)&move );
     else
-	XSendEvent( qt_xdisplay(), target, FALSE, NoEventMask,
+	XSendEvent( qt_xdisplay(), target, FALSE, emask,
 		    (XEvent*)&move );
 }
 
@@ -635,6 +766,9 @@ void QDragManager::drop()
     if ( !qt_xdnd_current_target )
 	return;
 
+    delete qt_xdnd_deco;
+    qt_xdnd_deco = 0;
+
     XClientMessageEvent drop;
     drop.type = ClientMessage;
     drop.window = qt_xdnd_current_target;
@@ -643,14 +777,21 @@ void QDragManager::drop()
     drop.data.l[0] = object->source()->winId();
     drop.data.l[1] = 1 << 24; // flags
     drop.data.l[2] = 0; // ###
-    drop.data.l[3] = 0;
+    drop.data.l[3] = qt_x_clipboardtime;
     drop.data.l[4] = 0;
 
     QWidget * w = QWidget::find( qt_xdnd_current_target );
+
+    int emask = NoEventMask;
+    if ( w && w->isDesktop() && !w->acceptDrops() ) {
+	emask = EnterWindowMask;
+	w = 0;
+    }
+
     if ( w )
 	qt_handle_xdnd_drop( w, (const XEvent *)&drop );
     else
-	XSendEvent( qt_xdisplay(), qt_xdnd_current_target, FALSE, NoEventMask,
+	XSendEvent( qt_xdisplay(), qt_xdnd_current_target, FALSE, emask,
 		    (XEvent*)&drop );
     qt_xdnd_current_target = 0;
     if ( restoreCursor ) {
@@ -667,6 +808,8 @@ bool qt_xdnd_handle_badwindow()
 	qt_xdnd_current_target = 0;
 	delete qt_xdnd_source_object;
 	qt_xdnd_source_object = 0;
+	delete qt_xdnd_deco;
+	qt_xdnd_deco = 0;
 	return TRUE;
     }
     if ( qt_xdnd_dragsource_xid ) {
@@ -682,6 +825,18 @@ bool qt_xdnd_handle_badwindow()
 }
 
 
+/*!
+  \class QDragMoveEvent qevent.h
+  \brief Event sent as a drag-and-drop is in progress.
+
+  When a widgets \link QWidget::setAcceptDrops() accepts drop events\endlink,
+  it will receive this event repeatedly while the the drag is inside that
+  widget.  The widget should examine the event, especially
+  seeing what data it \link QDragMoveEvent::provides provides\endlink,
+  and accept() the drop if appropriate.
+*/
+
+
 /*!  Returns a string describing one of the available data types for
   this drag.  Common examples are "text/plain" and "image/gif".  If \a
   n is less than zero or greater than the number of available data
@@ -765,8 +920,11 @@ static QByteArray qt_xdnd_obtain_data( const char * format )
 {
     QByteArray result;
 
+    QWidget* w;
     if ( qt_xdnd_dragsource_xid && qt_xdnd_source_object &&
-	 QWidget::find( qt_xdnd_dragsource_xid ) ) {
+	 (w=QWidget::find( qt_xdnd_dragsource_xid ))
+	   && (!w->isDesktop() || w->acceptDrops()) )
+    {
 	QDragObject * o = qt_xdnd_source_object;
 	if ( o->provides( format ) )
 	    result = o->encodedData(format);
@@ -783,43 +941,49 @@ static QByteArray qt_xdnd_obtain_data( const char * format )
     if ( qt_xdnd_target_data->find( (int)*a ) ) {
 	result = *(qt_xdnd_target_data->find( (int)*a ));
     } else {
-	if ( XGetSelectionOwner( qt_xdnd_current_widget->x11Display(),
+	if ( XGetSelectionOwner( qt_xdisplay(),
 				 qt_xdnd_selection ) == None )
 	    return result; // should never happen?
 
-	XConvertSelection( qt_xdnd_current_widget->x11Display(),
+	QWidget* tw = qt_xdnd_current_widget;
+	if ( !tw || qt_xdnd_current_widget->isDesktop() ) {
+	    tw = new QWidget;
+	}
+	XConvertSelection( qt_xdisplay(),
 			   qt_xdnd_selection, *a,
 			   qt_xdnd_selection,
-			   qt_xdnd_current_widget->winId(), CurrentTime );
-	XFlush( qt_xdnd_current_widget->x11Display() );
+			   tw->winId(), CurrentTime );
+	XFlush( qt_xdisplay() );
 
 	XEvent xevent;
-	if ( !qt_xclb_wait_for_event( qt_xdnd_current_widget->x11Display(),
-				      qt_xdnd_current_widget->winId(),
-				      SelectionNotify, &xevent, 5000) )
-	    return result;
+	bool got=qt_xclb_wait_for_event( qt_xdisplay(),
+				      tw->winId(),
+				      SelectionNotify, &xevent, 5000);
+	if ( got ) {
+	    Atom type;
 
-	Atom type;
-
-	if ( qt_xclb_read_property( qt_xdnd_current_widget->x11Display(),
-				    qt_xdnd_current_widget->winId(),
-				    qt_xdnd_selection, TRUE,
-				    &result, 0, &type, 0, FALSE ) ) {
-	    if ( type == None ) {
-		return result;
-	    } else if ( type == qt_incr_atom ) {
-		int nbytes = result.size() >= 4 ? *((int*)result.data()) : 0;
-		result = qt_xclb_read_incremental_property( qt_xdnd_current_widget->x11Display(),
-							    qt_xdnd_current_widget->winId(),
-							    qt_xdnd_selection,
-							    nbytes, FALSE );
-	    } else if ( type != *a ) {
-		//debug( "Qt clipboard: unknown atom %ld", type);
+	    if ( qt_xclb_read_property( qt_xdisplay(),
+					tw->winId(),
+					qt_xdnd_selection, TRUE,
+					&result, 0, &type, 0, FALSE ) ) {
+		if ( type == qt_incr_atom ) {
+		    int nbytes = result.size() >= 4 ? *((int*)result.data()) : 0;
+		    result = qt_xclb_read_incremental_property( qt_xdisplay(),
+								tw->winId(),
+								qt_xdnd_selection,
+								nbytes, FALSE );
+		} else if ( type != *a ) {
+		    // (includes None) debug( "Qt clipboard: unknown atom %ld", type);
+		}
+		if ( type != None )
+		    qt_xdnd_target_data->insert( (int)((long)a), new QByteArray(result) );
 	    }
 	}
-	qt_xdnd_target_data->insert( (int)((long)a), new QByteArray(result) );
+	if ( !qt_xdnd_current_widget || qt_xdnd_current_widget->isDesktop() ) {
+	    delete tw;
+	}
     }
-	
+
     return result;
 }
 
@@ -839,6 +1003,19 @@ QByteArray QDragMoveEvent::data( const char * format )
 }
 
 
+/*!
+  \class QDropEvent qevent.h
+  \brief Event sent when a drag-and-drop is completed.
+
+  When a widgets \link QWidget::setAcceptDrops() accepts drop events\endlink,
+  it will receive this event if it has accepted the most recent
+  QDragEnterEvent or QDragMoveEvent sent to it.
+
+  The widget should use data() to extract data in an
+  appropriate format.
+*/
+
+
 /*!  Returns a byte array containing the payload data of this drag, in
   \a format.
 
@@ -846,6 +1023,9 @@ QByteArray QDragMoveEvent::data( const char * format )
   potentially very slow, so it's advisable to call this function only
   if you're sure that you will need the data in \a format.
 
+  The resulting data will have a \link QByteArray::size() size\endlink
+  of 0 if the format was not available.
+
   \sa QDragMoveEvent::data() QDragMoveEvent::format()
 */
 
@@ -854,7 +1034,6 @@ QByteArray QDropEvent::data( const char * format )
     return qt_xdnd_obtain_data( format );
 }
 
-
 bool QDragManager::drag( QDragObject * o, QDragObject::DragMode )
 {
     if ( object == o )
@@ -867,6 +1046,7 @@ bool QDragManager::drag( QDragObject * o, QDragObject::DragMode )
     }
 
     qt_xdnd_source_object = o;
+    qt_xdnd_deco = new QShapedPixmapWidget();
 
     willDrop = FALSE;
 
@@ -877,7 +1057,31 @@ bool QDragManager::drag( QDragObject * o, QDragObject::DragMode )
     XSetSelectionOwner( qt_xdisplay(), qt_xdnd_selection,
 			dragSource->topLevelWidget()->winId(),
 			qt_xdnd_source_current_time );
+    updatePixmap();
+    move(QCursor::pos());
 
     qApp->enter_loop();
+
+    delete qt_xdnd_deco;
+    qt_xdnd_deco = 0;
+    if ( dragSource && qt_xdnd_source_object ) {
+	dragSource->removeChild(qt_xdnd_source_object);
+	// qt_xdnd_source_object will become 0 later.
+    }
+
     return FALSE;
 }
+
+void QDragManager::updatePixmap()
+{
+    if ( qt_xdnd_deco ) {
+	if ( object && !object->pixmap().isNull() ) {
+	    qt_xdnd_deco->setPixmap(object->pixmap());
+	    qt_xdnd_deco->move(QCursor::pos()-qt_xdnd_source_object->pixmapHotSpot());
+	    //qt_xdnd_deco->repaint(FALSE);
+	    qt_xdnd_deco->show();
+	} else if ( qt_xdnd_deco ) {
+	    qt_xdnd_deco->hide();
+	}
+    }
+}
