@@ -1,41 +1,43 @@
-/****************************************************************************
-** $Id: qobject.cpp,v 2.32 1997/10/08 16:56:22 warwick Exp $
+ /****************************************************************************
+** $Id: qobject.cpp,v 2.52.2.5 1998/11/20 12:28:28 warwick Exp $
 **
 ** Implementation of QObject class
 **
 ** Created : 930418
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qobject.h"
-#include "qobjcoll.h"
+#include "qobjectlist.h"
+#include "qobjectdict.h"
+#include "qsignalslotimp.h"
 #include "qregexp.h"
 #include <ctype.h>
 #if defined(DEBUG)
-#include "qapp.h"
+#include "qapplication.h"
 #endif
 
-RCSTAG("$Id: qobject.cpp,v 2.32 1997/10/08 16:56:22 warwick Exp $");
-
-
 /*!
   \class QObject qobject.h
   \brief The QObject class is the base class of all Qt objects that can
   deal with signals, slots and events.
 
+  \ingroup kernel
+
   Qt provides a very powerful mechanism for seamless object
   communication; \link metaobjects.html signal/slot
   connections\endlink. The signal/slot mechanism is an advanced way
@@ -142,6 +144,8 @@ RCSTAG("$Id: qobject.cpp,v 2.32 1997/10/08 16:56:22 warwick Exp $");
   \class QSenderObject qobject.h
   \brief Internal object used for sending signals.
 
+  \internal
+
   It is generally a very bad idea to use this class directly in
   application programs.
 
@@ -163,12 +167,13 @@ RCSTAG("$Id: qobject.cpp,v 2.32 1997/10/08 16:56:22 warwick Exp $");
 // Internal for QObject::connect() and QObject::disconnect()
 //
 
-inline bool isIdentChar( char x )
-{
-    return isalnum(x) || x == '_';
+static inline bool isIdentChar( char x )
+{						// Avoid bug in isalnum
+    return x == '_' || (x >= '0' && x <= '9') ||
+	 (x >= 'a' && x <= 'z') || (x >= 'A' && x <= 'Z');
 }
 
-inline bool isSpace( char x )
+static inline bool isSpace( char x )
 {
 #if defined(_CC_BOR_)
   /*
@@ -199,6 +204,9 @@ static QString rmWS( const char *src )
 	    last = *d++ = ' ';
     }
     result.truncate( (int)(d - result.data()) );
+    int void_pos = result.find("(void)");
+    if ( void_pos >= 0 )
+	result.remove( void_pos+1, strlen("void") );
     return result;
 }
 
@@ -212,10 +220,10 @@ bool  qKillTimer( QObject *obj );
 void  qRemovePostedEvents( QObject * );
 
 
-Q_DECLARE(QListM,QConnection);			// list of connections
-Q_DECLARE(QListIteratorM,QConnection);		// connection list iterator
+#if !defined(QT_MAKEDLL)
 Q_DECLARE(QDictM,QListM_QConnection);		// dict for multiple signals
 Q_DECLARE(QDictIteratorM,QListM_QConnection);	// dict iterator
+#endif
 
 QMetaObject *QObject::metaObj = 0;
 
@@ -243,7 +251,7 @@ static void removeObjFromList( QObjectList *objList, const QObject *obj,
 
   Returns 0 if there is no such child.
 
-  The CHILD macro (defined in qwindefs.h) does all this, and also
+  The CHILD macro (defined in qwindowdefs.h) does all this, and also
   casts the returt type to \a type *.
 
   Examples:
@@ -269,7 +277,7 @@ void *qt_find_obj_child( QObject *parent, const char *type, const char *name )
 	QObject *obj;
 	while ( (obj=it.current()) ) {
 	    ++it;
-	    if ( strcmp(name,obj->name()) == 0 &&
+	    if ( qstrcmp(name,obj->name()) == 0 &&
 		obj->inherits(type) )
 		return obj;
 	}
@@ -405,11 +413,9 @@ QObject::~QObject()
   mechanism.  The functions isA() and inherits() also make use of the
   meta object.
 
-  The meta object is created by the initMetaObject() function, which is
-  generated by the meta object compiler and called from the class constructor.
-
-  \warning If this function returns 0, the constructor probably forgot
-  to call initMetaObject().
+  The meta object is created by the initMetaObject() function, which
+  is generated by the meta object compiler and called from the QObject
+  constructor.
 */
 
 /*!
@@ -419,8 +425,7 @@ QObject::~QObject()
   Compiler. \endlink
 
   \warning This function will return an invalid name if the class
-  constructor did not call initMetaObject() or if the class definition
-  lacks the \c Q_OBJECT macro.
+  definition lacks the \c Q_OBJECT macro.
 
   \sa name(), inherits(), isA(), isWidgetType()
 */
@@ -453,7 +458,9 @@ bool QObject::isA( const char *clname ) const
 
 /*!
   Returns TRUE if this object is an instance of a class that inherits
-  \e clname.  (A class is considered to inherit itself.)
+  \e clname, and \a clname inherits QObject.
+
+  (A class is considered to inherit itself.)
 
   Example:
   \code
@@ -461,6 +468,10 @@ bool QObject::isA( const char *clname ) const
     t->inherits("QTimer");		// returns TRUE
     t->inherits("QObject");		// returns TRUE
     t->inherits("QButton");		// returns FALSE
+
+    QScrollBar * s = new QScrollBar;	// inherits QWidget and QRangeControl
+    s->inherits( "QWidget" );		// returns TRUE
+    s->inherits( "QRangeControl" ); 	// returns FALSE
   \endcode
 
   \sa isA(), metaObject()
@@ -494,6 +505,23 @@ bool QObject::inherits( const char *clname ) const
   \sa setName(), className(), queryList()
 */
 
+/*!
+  \overload const char *QObject::name( const char * defaultName ) const
+
+  Returns the name of this object, or \a defaultName if the object
+  does not have a name.
+
+  This version of name() is particularly useful for debug() calls such as:
+
+  \code
+    debug( "MyClass::setPrecision(): (%s) unable to set precision to %f",
+	    name( "unnamed" ), newPrecision );
+  \endcode
+
+  name( "unnamed" ) is used here so that printf() (used in debug())
+  will not be asked to output a null pointer.
+*/
+
 /*!
   Sets the name of this object to \e name.  The default name is the
   one assigned by the constructor.
@@ -566,6 +594,9 @@ bool QObject::event( QEvent *e )
 
   The default implementation does nothing.
 
+  QTimer provides a higher-level interface to the timer functionality,
+  and also more general information about timers.
+
   \sa startTimer(), killTimer(), killTimers(), event()
 */
 
@@ -604,8 +635,9 @@ bool QObject::activate_filters( QEvent *e )
     register QObject *obj = it.current();
     while ( obj ) {				// send to all filters
 	++it;					//   until one returns TRUE
-	if ( obj->eventFilter(this,e) )
+	if ( obj->eventFilter(this,e) ) {
 	    return TRUE;
+	}
 	obj = it.current();
     }
     return FALSE;				// don't do anything with it
@@ -643,7 +675,7 @@ void QObject::blockSignals( bool block )
 
   A timer event will occur every \e interval milliseconds until
   killTimer() or killTimers() is called.  If \e interval is 0, then
-  timer event occurs once every time there are no more window system
+  the timer event occurs once every time there are no more window system
   events to process.
 
   The virtual timerEvent() function is called with the QTimerEvent event
@@ -741,7 +773,7 @@ static void objSearch( QObjectList *result,
 	    ok = FALSE;
 	if ( ok ) {
 	    if ( objName )
-		ok = strcmp(objName,obj->name()) == 0;
+		ok = qstrcmp(objName,obj->name()) == 0;
 	    else if ( rx )
 		ok = rx->match(obj->name()) >= 0;
 	}
@@ -765,6 +797,8 @@ static void objSearch( QObjectList *result,
   \fn const QObjectList *QObject::children() const
   Returns a list of child objects, or 0 if this object has no children.
 
+  The QObjectList class is defined in the qobjcoll.h header file.
+
   The latest child added is the \link QList::first() first\endlink object
   in the list and the first child is added is the \link QList::last()
   last\endlink object in the list.
@@ -805,7 +839,7 @@ static void objSearch( QObjectList *result,
     QObject * obj;
     while ( (obj=it.current()) != 0 ) {	// for each found object...
 	++it;
-	obj->setFont( newFont );
+	((QButton*)obj)->setFont( newFont );
     }
     delete list;			// delete the list, not the objects
   \endcode
@@ -892,7 +926,7 @@ void QObject::insertChild( QObject *obj )
 #if defined(CHECK_STATE)
     else if ( childObjects->findRef(obj) >= 0 ) {
 	warning( "QObject::insertChild: Object %s::%s already in list",
-		 obj->className(), obj->name() );
+		 obj->className(), obj->name( "unnamed" ) );
 	return;
     }
 #endif
@@ -1062,6 +1096,19 @@ static void err_member_notfound( int code, const QObject *object,
 		 func, type, object->className(), member );
 }
 
+
+static void err_info_about_objects( const char * func,
+				    const QObject * sender,
+				    const QObject * receiver )
+{
+    const char * a = sender->name(), * b = receiver->name();
+    if ( a )
+	warning( "QObject::%s:  (sender name:   '%s')", func, a );
+    if ( b )
+	warning( "QObject::%s:  (receiver name: '%s')", func, b );
+}
+
+
 #endif // CHECK_RANGE
 
 
@@ -1275,6 +1322,7 @@ bool QObject::connect( const QObject *sender,	const char *signal,
     if ( !(sm=smeta->signal(signal,TRUE)) ) {	// no such signal
 #if defined(CHECK_RANGE)
 	err_member_notfound( SIGNAL_CODE, sender, signal, "connect" );
+	err_info_about_objects( "connect", sender, receiver );
 #endif
 	return FALSE;
     }
@@ -1301,6 +1349,7 @@ bool QObject::connect( const QObject *sender,	const char *signal,
     if ( !rm ) {				// no such member
 #if defined(CHECK_RANGE)
 	err_member_notfound( membcode, r, member, "connect" );
+	err_info_about_objects( "connect", sender, receiver );
 #endif
 	return FALSE;
     }
@@ -1354,6 +1403,10 @@ bool QObject::connect( const QObject *sender,	const char *signal,
   involved are destroyed.
 */
 
+#if (QT_VERSION >= 200)
+#error "disconnect const foo"
+#endif
+
 /*!
   Disconnects \e signal in object \e sender from \e member in object \e
   receiver.
@@ -1436,6 +1489,7 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	if ( !rm ) {				// no such member
 #if defined(CHECK_RANGE)
 	    err_member_notfound( membcode, r, member, "disconnect" );
+	    err_info_about_objects( "disconnect", sender, receiver );
 #endif
 	    return FALSE;
 	}
@@ -1538,8 +1592,8 @@ void QObject::cleanupEventFilter()
 
 /*!
   \internal
-  Returns the meta object for this object. If there is no meta object,
-  it calls initMetaObject() and makes another try.
+  Returns the meta object for this object. If necessary, calls
+  initMetaObject().
   \sa metaObject()
 */
 
@@ -1554,14 +1608,15 @@ QMetaObject *QObject::queryMetaObject() const
 #if defined(CHECK_NULL)
     if ( !m )					// still no meta object: error
 	warning( "QObject: Object %s::%s has no meta object",
-		 x->className(), x->name() );
+		 x->className(), x->name( "unnamed" ) );
 #endif
     return m;
 }
 
 /*!
-  Initializes the \link metaObject() meta object\endlink of this object.
-
+  Initializes the \link metaObject() meta object\endlink of this
+  object. This method is automatically executed on demand from the
+  QObject constructor.
   \sa metaObject()
 */
 void QObject::initMetaObject()
@@ -1677,10 +1732,21 @@ static void dumpRecursive( int level, QObject *object )
     if ( object ) {
 	QString buf;
 	buf.fill( '\t', level );
-	const char *name = object->name() ? object->name() : "????";
+	const char *name = object->name( "unnamed" );
 	QString flags="";
 	if ( qApp->focusWidget() == object )
 	    flags += 'F';
+	if ( object->isWidgetType() ) {
+	    QWidget * w = (QWidget *)object;
+	    if ( w->isVisible() ) {
+		QString t;
+		t.sprintf( "<%d,%d,%d,%d>",
+			   w->x(), w->y(), w->width(), w->height() );
+		flags += t;
+	    } else {
+		flags += 'I';
+	    }
+	}	
 	debug( "%s%s::%s %s", (const char*)buf, object->className(), name,
 	    (const char*)flags );
 	if ( object->children() ) {
@@ -1715,7 +1781,7 @@ void QObject::dumpObjectTree()
 void QObject::dumpObjectInfo()
 {
 #if defined(DEBUG)
-    debug( "OBJECT %s::%s", className(), name() );
+    debug( "OBJECT %s::%s", className(), name( "unnamed" ) );
     debug( "  SIGNALS OUT" );
     int n = 0;
     if ( connections ) {
@@ -1730,7 +1796,7 @@ void QObject::dumpObjectInfo()
 	    while ( (c=cit.current()) ) {
 		++cit;
 		debug( "\t  --> %s::%s %s", c->object()->className(),
-		       c->object()->name(), c->memberName() );
+		       c->object()->name( "unnamed" ), c->memberName() );
 	    }
 	}
     }
@@ -1741,7 +1807,8 @@ void QObject::dumpObjectInfo()
     if ( senderObjects ) {
 	QObject *sender = senderObjects->first();
 	while ( sender ) {
-	    debug( "\t%s::%s", sender->className(), sender->name() );
+	    debug( "\t%s::%s",
+		   sender->className(), sender->name( "unnamed" ) );
 	    n++;
 	    sender = senderObjects->next();
 	}
@@ -1750,3 +1817,14 @@ void QObject::dumpObjectInfo()
 	debug( "\t<None>" );
 #endif
 }
+
+
+
+/*!  Returns a translated version of \a text, or \a text if there is
+  no appropriate translated version.
+*/
+
+const char * QObject::tr( const char * text ) const
+{
+    return text;
+}
