@@ -1,34 +1,37 @@
 /****************************************************************************
-** $Id: qobject.h,v 2.5 1997/08/06 15:05:39 hanord Exp $
+** $Id: qobject.h,v 2.11.2.3 1998/10/19 12:08:19 eiriken Exp $
 **
 ** Definition of QObject class
 **
 ** Created : 930418
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QOBJECT_H
 #define QOBJECT_H
 
-#include "qobjdefs.h"
+#ifndef QT_H
+#include "qobjectdefs.h"
 #include "qstring.h"
 #include "qevent.h"
+#endif // QT_H
 
 
-class QObject					// base class for Q objects
+class Q_EXPORT QObject					// base class for Q objects
 {
 public:
     QObject( QObject *parent=0, const char *name=0 );
@@ -37,6 +40,8 @@ public:
     virtual bool event( QEvent * );
     virtual bool eventFilter( QObject *, QEvent * );
 
+    const char * tr( const char * ) const;
+
     virtual QMetaObject *metaObject() const { return metaObj; }
     virtual const char	*className()  const;
 
@@ -44,6 +49,10 @@ public:
     bool	inherits( const char * ) const;
 
     const char *name()		  const { return objname; }
+    const char *name( const char * defaultName ) const { return objname
+							     ? objname
+							     : defaultName; }
+
     void	setName( const char *name );
     bool	isWidgetType()	  const { return isWidget; }
     bool	highPriority()	  const { return hiPriority; }
@@ -101,11 +110,11 @@ protected:
     virtual void timerEvent( QTimerEvent * );
 
     virtual void connectNotify( const char *signal );
-    virtual void disconnectNotify( const char *signal );
+   virtual void disconnectNotify( const char *signal );
     virtual bool checkConnectArgs( const char *signal, const QObject *receiver,
 				   const char *member );
 
-    static  void badSuperclassWarning( const char *className, 
+    static  void badSuperclassWarning( const char *className,
 				       const char *superclassName );
 
     uint	isSignal   : 1;
@@ -135,8 +144,10 @@ private:
     friend class QSenderObject;
 
 private:	// Disabled copy constructor and operator=
-    QObject( const QObject & ) {}
-    QObject &operator=( const QObject & ) { return *this; }
+#if defined(Q_DISABLE_COPY)
+    QObject( const QObject & );
+    QObject &operator=( const QObject & );
+#endif
 };
 
 
@@ -163,7 +174,7 @@ inline const QObject *QObject::sender()
 }
 
 
-class QSenderObject : public QObject		// object for sending signals
+class Q_EXPORT QSenderObject : public QObject		// object for sending signals
 {
 public:
     void setSender( QObject *s ) { sigSender=s; }
