@@ -1,46 +1,39 @@
 /**********************************************************************
-** $Id: qprinter.cpp,v 2.3 1997/08/25 16:10:23 eiriken Exp $
+** $Id: qprinter.cpp,v 2.10.2.3 1998/11/02 12:58:39 paul Exp $
 **
 ** Implementation of QPrinter class
 **
 ** Created : 941003
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qprinter.h"
 
-RCSTAG("$Id: qprinter.cpp,v 2.3 1997/08/25 16:10:23 eiriken Exp $");
-
-
 /*!
   \class QPrinter qprinter.h
   \brief The QPrinter class is a paint device that prints graphics on a
   printer.
 
-  \ingroup paintdevice
+  \ingroup drawing
 
   All window systems that Qt supports, except X11, have built-in
   printer drivers.  For X11, Qt provides PostScript (tm)
   printing.
 
-  Clipping and even-odd filling are not supported.
-
-  Font support is limited to Times, Helvetica, Courier and Symbol, and
-  font metrics for these four fonts can be inaccurate.
-
   Drawing graphics on a printer is almost identical to drawing graphics
   in a widget or a pixmap.  The only difference is that the programmer
   must think about dividing the document into pages and handling abort
@@ -64,16 +57,17 @@ RCSTAG("$Id: qprinter.cpp,v 2.3 1997/08/25 16:10:23 eiriken Exp $");
   Example (a complete application):
 
   \code
-    #include <qapp.h>
+    #include <qapplication.h>
     #include <qpainter.h>
     #include <qprinter.h>
+    #include <qprintdialog.h>
 
     int main( int argc, char **argv )
     {
 	QApplication a( argc, argv );
 
 	QPrinter prt;
-	if ( prt.setup(0) ) {
+	if ( prt.setup() ) {
 	    QPainter p;
 	    p.begin( &prt );
 	    p.rotate( 55 );
@@ -273,7 +267,7 @@ void QPrinter::setCreator( const char *creator )
   Sets the print orientation.
 
   The orientation can be either \c QPrinter::Portrait or
-  \c QPainter::Landscape.
+  \c QPrinter::Landscape.
 
   The printer driver reads this setting and prints using the specified
   orientation.
@@ -293,24 +287,123 @@ void QPrinter::setOrientation( Orientation orientation )
   \sa setPageSize()
 */
 
-/*!
-  Sets the printer page size.
 
-  The page size can be one of:
+static QPrinter::PageSize makepagesize( QPrinter::PageSize ps,
+					QPrinter::PageOrder po,
+					QPrinter::ColorMode cm )
+{
+    return (QPrinter::PageSize)( ((int)ps & 255) +
+				 ((po == QPrinter::LastPageFirst) ? 256 : 0) +
+				 ((cm == QPrinter::GrayScale) ? 512 : 0) );
+}
+
+
+
+//   <li>\c QPrinter::Executive (184 x 267 mm)
+
+/*!
+  Sets the printer page size to \a newPageSize.
+
+  The page size can be one of
   <ul>
+  <li>\c QPrinter::A0 (841 x 1189 mm)
+  <li>\c QPrinter::A1 (594 x 841 mm)
+  <li>\c QPrinter::A2 (420 x 594 mm)
+  <li>\c QPrinter::A3 (297 x 420 mm)
   <li>\c QPrinter::A4 (210x297 mm, 8.26x11.7 inches)
+  <li>\c QPrinter::A5 (148 x 210 mm)
+  <li>\c QPrinter::A6 (105 x 148 mm)
+  <li>\c QPrinter::A7 (74 x 105 mm)
+  <li>\c QPrinter::A8 (52 x 74 mm)
+  <li>\c QPrinter::A9 (37 x 52 mm)
+  <li>\c QPrinter::B0 (1030 x 1456 mm)
+  <li>\c QPrinter::B1 (728 x 1030 mm)
+  <li>\c QPrinter::B10 (32 x 45 mm)
+  <li>\c QPrinter::B2 (515 x 728 mm)
+  <li>\c QPrinter::B3 (364 x 515 mm)
+  <li>\c QPrinter::B4 (257 x 364 mm)
   <li>\c QPrinter::B5 (182x257 mm, 7.17x10.13 inches)
-  <li>\c QPrinter::Letter (8.5x11 inches, 216x279 mm)
-  <li>\c QPrinter::Legal ( 8.5x14 inches, 216x356 mm)
+  <li>\c QPrinter::B6 (128 x 182 mm)
+  <li>\c QPrinter::B7 (91 x 128 mm)
+  <li>\c QPrinter::B8 (64 x 91 mm)
+  <li>\c QPrinter::B9 (45 x 64 mm)
+  <li>\c QPrinter::C5E (163 x 229 mm)
+  <li>\c QPrinter::Comm10E (105 x 241 mm, US Common #10 Envelope)
+  <li>\c QPrinter::DLE (110 x 220 mm)
   <li>\c QPrinter::Executive (7.5x10 inches, 191x254 mm)
+  <li>\c QPrinter::Folio (210 x 330 mm)
+  <li>\c QPrinter::Ledger (432 x 279 mm)
+  <li>\c QPrinter::Legal (8.5x14 inches, 216x356 mm)
+  <li>\c QPrinter::Letter (8.5x11 inches, 216x279 mm)
+  <li>\c QPrinter::Tabloid (279 x 432 mm)
+
   </ul>
 
   \sa pageSize()
 */
 
-void QPrinter::setPageSize( PageSize pageSize )
+void QPrinter::setPageSize( PageSize newPageSize )
 {
-    page_size = pageSize;
+    page_size = makepagesize( newPageSize, pageOrder(), colorMode() );
+}
+
+
+/*!  Sets the page order to \a newPageOrder.
+
+  The page order can be \c QPrinter::FirstPageFirst or \c
+  QPrinter::LastPageFirst.  The application programmer is responsible
+  for reading the page order and printing accordingly.
+*/
+
+void QPrinter::setPageOrder( PageOrder newPageOrder )
+{
+    page_size = makepagesize( pageSize(), newPageOrder, colorMode() );
+}
+
+
+/*!  Returns the current page order.
+
+  The default page order is \a FirstPageFirst.
+*/
+
+QPrinter::PageOrder QPrinter::pageOrder() const
+{
+    if ( ((int)page_size) & 256 )
+	return QPrinter::LastPageFirst;
+    else
+	return QPrinter::FirstPageFirst;
+}
+
+
+/*!  Sets the printer's color mode to \a newColorMode, which can be
+  one of \c Color (the default) and \c GrayScale.
+
+  A future version of Qt will modify its printing accordingly.  At
+  present, QPrinter behaves as if \c Color is selected.
+
+  \sa colorMode()
+*/
+
+void QPrinter::setColorMode( ColorMode newColorMode )
+{
+    page_size = makepagesize( pageSize(), pageOrder(), newColorMode );
+}
+
+
+/*!  Returns the current color mode.  The default color more is \c
+  Color.
+
+  \sa setColorMode()
+*/
+
+QPrinter::ColorMode QPrinter::colorMode() const
+{
+    if ( ((int)page_size) & 512 )
+	return QPrinter::GrayScale;
+    else
+	return QPrinter::Color;
+
+
 }
 
 
