@@ -1,41 +1,41 @@
 /****************************************************************************
-** $Id: qrect.cpp,v 2.4 1997/09/04 12:33:31 warwick Exp $
+** $Id: qrect.cpp,v 2.9 1998/07/03 00:09:40 hanord Exp $
 **
 ** Implementation of QRect class
 **
 ** Created : 931028
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #define	 QRECT_C
 #include "qrect.h"
-#include "qdstream.h"
-
-RCSTAG("$Id: qrect.cpp,v 2.4 1997/09/04 12:33:31 warwick Exp $");
-
+#include "qdatastream.h"
 
 /*!
   \class QRect qrect.h
   \brief The QRect class defines a rectangle in the plane.
 
+  \ingroup drawing
+
   A rectangle is internally represented as an upper left corner and a
   bottom right corner, but it is normally expressed as an upper left
   corner and a size.
 
-  The coordinate type is QCOORD (defined in qwindefs.h as \c short).
+  The coordinate type is QCOORD (defined in qwindowdefs.h as \c short).
   The minimum value of QCOORD is QCOORD_MIN (-32768) and the maximum
   value is  QCOORD_MAX (32767).
 
@@ -561,8 +561,10 @@ bool QRect::contains( const QRect &r, bool proper ) const
 }
 
 /*!
-  Returns the union rectangle of this rectangle and \e r.
-  If \e r is empty, this rectangle is returned unchanged.
+  Returns the union rectangle of this rectangle and \e r.  The union
+  rectangle of a nonempty rectangle and an empty or invalid rectangle
+  is defined to be the nonempty rectangle.
+
   \sa intersect(), intersects(), contains()
 */
 
