@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qarray.h,v 2.1 1997/08/06 15:05:49 hanord Exp $
+** $Id: qarray.h,v 2.3.2.3 1998/08/27 08:14:00 hanord Exp $
 **
 ** Definition of QArray template/macro class
 **
 ** Created : 930906
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QARRAY_H
 #define QARRAY_H
 
+#ifndef QT_H
 #include "qgarray.h"
+#endif // QT_H
 
 
 #if defined(USE_MACROCLASS)
@@ -42,7 +45,7 @@
 #define QArrayM(type) name2(QArrayM_,type)
 
 #define QArrayMdeclare(type)						      \
-class QArrayM(type) : public QGArray					      \
+class Q_EXPORTM QArrayM(type) : public QGArray				      \
 {									      \
 protected:								      \
     QArrayM(type)( int, int ) : QGArray( 0, 0 ) {}			      \
@@ -101,7 +104,7 @@ public:									      \
 #define QArray QArrayT
 #endif
 
-template<class type> class QArrayT : public QGArray
+template<class type> class Q_EXPORT QArrayT : public QGArray
 {
 protected:
     QArrayT( int, int ) : QGArray( 0, 0 ) {}
