@@ -1,34 +1,34 @@
 /**********************************************************************
-** $Id: qlistbox.cpp,v 2.49.2.2 1997/11/24 12:16:47 paul Exp $
+** $Id: qlistbox.cpp,v 2.71.2.11 1999/02/13 15:50:58 hanord Exp $
 **
 ** Implementation of QListBox widget class
 **
 ** Created : 941121
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qlistbox.h"
-#include "qfontmet.h"
+#include "qfontmetrics.h"
 #include "qpainter.h"
 #include "qstrlist.h"
 #include "qkeycode.h"
+#include "qscrollbar.h"
 #include "qpixmap.h"
-#include "qapp.h"
-
-RCSTAG("$Id: qlistbox.cpp,v 2.49.2.2 1997/11/24 12:16:47 paul Exp $");
+#include "qapplication.h"
 
 Q_DECLARE(QListM, QListBoxItem);
 
@@ -47,24 +47,28 @@ int QLBItemList::compareItems( GCI i1, GCI i2)
 }
 
 
-static inline bool checkInsertIndex( const char *method, int count, int *index)
+static inline bool checkInsertIndex( const char *method, const char * name,
+				     int count, int *index)
 {
     bool range_err = (*index > count);
 #if defined(CHECK_RANGE)
     if ( *index > count )
-	warning( "QListBox::%s: Index %i out of range", method, *index );
+	warning( "QListBox::%s: (%s) Index %i out of range",
+		 method, name ? name : "<no name>", *index );
 #endif
     if ( *index < 0 )				// append
 	*index = count;
     return !range_err;
 }
 
-static inline bool checkIndex( const char *method, int count, int index )
+static inline bool checkIndex( const char *method, const char * name,
+			       int count, int index )
 {
     bool range_err = ((uint)index >= (uint)count);
 #if defined(CHECK_RANGE)
     if ( range_err )
-	warning( "QListBox::%s: Index %d out of range", method, index );
+	warning( "QListBox::%s: (%s) Index %d out of range",
+		 method, name ? name : "<no name>", index );
 #endif
     return !range_err;
 }
@@ -74,8 +78,6 @@ static inline bool checkIndex( const char *method, int count, int index )
   \class QListBoxItem qlistbox.h
   \brief This is the base class of all list box items.
 
-  \ingroup abstract
-
   This class is the abstract base class of all list box items. If you
   need to insert customized items into a QListBox, you must inherit
   this class and reimplement paint(), height() and width().
@@ -198,7 +200,7 @@ QListBoxItem::~QListBoxItem()
 
 /*!
   \class QListBoxText qlistbox.h
-  \brief QListBoxText provides list box items with text.
+  \brief The QListBoxText class provides list box items with text.
 
   The text is drawn in the widget's current font. If you need several
   different fonts, you have to make your own subclass of QListBoxItem.
@@ -264,7 +266,7 @@ int QListBoxText::width( const QListBox *lb ) const
 
 /*!
   \class QListBoxPixmap qlistbox.h
-  \brief QListBoxPixmap provides list box items with a pixmap.
+  \brief The QListBoxPixmap class provides list box items with a pixmap.
 
   \sa QListBox, QListBoxItem
 */
@@ -342,7 +344,7 @@ int QListBoxPixmap::width( const QListBox * ) const
   If the user does not select anything, no signals are emitted and
   currentItem() returns -1.
 
-  A list box has \c StrongFocus as a default focusPolicy(), i.e. it can 
+  A list box has \c StrongFocus as a default focusPolicy(), i.e. it can
   get keyboard focus both by tabbing and clicking.
 
   New items may be inserted using either insertItem(), insertStrList()
@@ -374,6 +376,10 @@ int QListBoxPixmap::width( const QListBox * ) const
   and will delete them when they are not needed.
 
   <img src=qlistbox-m.gif> <img src=qlistbox-w.gif>
+
+  \sa QListView QComboBox QButtonGroup
+  <a href="guibooks.html#fowler">GUI Design Handbook: List Box (two
+  sections)</a>
 */
 
 
@@ -384,7 +390,7 @@ int QListBoxPixmap::width( const QListBox * ) const
 #error "Remove QListBox dict."
 #endif
 
-#include <qintdict.h>
+#include "qintdict.h"
 
 static QIntDict<int> *qlb_maxLenDict = 0;
 
@@ -408,7 +414,6 @@ static void cleanupListbox()
 QListBox::QListBox( QWidget *parent, const char *name, WFlags f )
     : QTableView( parent, name, f )
 {
-    initMetaObject();
     doDrag	  = TRUE;
     doAutoScroll  = TRUE;
     current	  = -1;
@@ -443,7 +448,6 @@ QListBox::QListBox( QWidget *parent, const char *name, WFlags f )
     }
 }
 
-
 #if QT_VERSION == 200
 #error "Remove QListBox pointer."
 #endif
@@ -471,7 +475,7 @@ QListBox::~QListBox()
   listbox changes. If the user selects five items by drag-selecting,
   QListBox tries to emit just one selectionChanged() signal, so the
   signal can be connected to computationally expensive slots.
-  
+
   \sa selected() currentItem()
 */
 
@@ -487,7 +491,7 @@ QListBox::~QListBox()
 
   This signal is emitted when the user highlights a new current item
   and the new item is a string.  The argument is the text of the
-  new current item.  
+  new current item.
 
   \sa selected() currentItem() selectionChanged()
 */
@@ -512,7 +516,8 @@ QListBox::~QListBox()
 */
 
 /*!
-  Reimplements QWidget::setFont() to update the list box line height.
+  Reimplements QWidget::setFont() to update the list box line height
+  and maxItemWidth().
 */
 
 void QListBox::setFont( const QFont &font )
@@ -520,6 +525,9 @@ void QListBox::setFont( const QFont &font )
     QWidget::setFont( font );
     if ( stringsOnly )
 	setCellHeight( fontMetrics().lineSpacing() + 1 );
+    // else ...?
+
+    updateCellWidth();
 }
 
 
@@ -544,7 +552,7 @@ uint QListBox::count() const
 
 void QListBox::insertStrList( const QStrList *list, int index )
 {
-    if ( !checkInsertIndex( "insertStrList", count(), &index ) )
+    if ( !checkInsertIndex( "insertStrList", name(), count(), &index ) )
 	return;
     if ( !list ) {
 #if defined(CHECK_NULL)
@@ -579,7 +587,7 @@ void QListBox::insertStrList( const QStrList *list, int index )
 
 void QListBox::insertStrList( const char **strings, int numStrings, int index )
 {
-    if ( !checkInsertIndex( "insertStrList", count(), &index ) )
+    if ( !checkInsertIndex( "insertStrList", name(), count(), &index ) )
 	return;
     if ( !strings ) {
 #if defined(CHECK_NULL)
@@ -613,7 +621,7 @@ void QListBox::insertStrList( const char **strings, int numStrings, int index )
 
 void QListBox::insertItem( const QListBoxItem *lbi, int index )
 {
-    if ( !checkInsertIndex( "insertItem", count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
 	return;
     if ( !lbi ) {
 #if defined ( CHECK_NULL )
@@ -643,7 +651,7 @@ void QListBox::insertItem( const QListBoxItem *lbi, int index )
 
 void QListBox::insertItem( const char *text, int index )
 {
-    if ( !checkInsertIndex( "insertItem", count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
 	return;
     if ( !text ) {
 #if defined ( CHECK_NULL )
@@ -673,7 +681,7 @@ void QListBox::insertItem( const char *text, int index )
 
 void QListBox::insertItem( const QPixmap &pixmap, int index )
 {
-    if ( !checkInsertIndex( "insertItem", count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
 	return;
     if ( stringsOnly ) {
 	stringsOnly = FALSE;
@@ -747,11 +755,11 @@ void QListBox::inSort( const char *text )
 
 void QListBox::removeItem( int index )
 {
-    if ( !checkIndex( "removeItem", count(), index ) )
+    if ( !checkIndex( "removeItem", name(), count(), index ) )
 	return;
     bool currentChanged = ( current == index );
 
-    if ( current >= index && current > 0 )
+    if( current > 0 && (current > index || current >= (int)count()-1) )
 	current--;
     bool    updt = autoUpdate() && itemVisible( index );
     QListBoxItem *lbi = itemList->take( index );
@@ -822,7 +830,7 @@ const QPixmap *QListBox::pixmap( int index ) const
 
 void QListBox::changeItem( const char *text, int index )
 {
-    if ( !checkIndex( "changeItem", count(), index ) )
+    if ( !checkIndex( "changeItem", name(), count(), index ) )
 	return;
     change( new QListBoxText(text), index );
 }
@@ -837,7 +845,7 @@ void QListBox::changeItem( const char *text, int index )
 
 void QListBox::changeItem( const QPixmap &pixmap, int index )
 {
-    if ( !checkIndex( "changeItem", count(), index ) )
+    if ( !checkIndex( "changeItem", name(), count(), index ) )
 	return;
     change( new QListBoxPixmap(pixmap), index );
 }
@@ -852,7 +860,7 @@ void QListBox::changeItem( const QPixmap &pixmap, int index )
 
 void QListBox::changeItem( const QListBoxItem *lbi, int index )
 {
-    if ( !checkIndex( "changeItem", count(), index ) )
+    if ( !checkIndex( "changeItem", name(), count(), index ) )
 	return;
     change( lbi, index );
 }
@@ -896,6 +904,8 @@ void QListBox::setAutoUpdate( bool enable )
 /*!
   Returns the number of visible items.	This may change at any time
   since the user may resize the widget.
+
+  \sa setFixedVisibleLines()
 */
 
 int QListBox::numItemsVisible() const
@@ -925,7 +935,7 @@ void QListBox::setCurrentItem( int index )
 {
     if ( index == current )
 	return;
-    if ( !checkIndex( "setCurrentItem", count(), index ) )
+    if ( !checkIndex( "setCurrentItem", name(), count(), index ) )
 	return;
     int oldCurrent = current;
     current	   = index;
@@ -981,6 +991,21 @@ void QListBox::setTopItem( int index )
     setTopCell( index );
 }
 
+/*!
+  Scrolls the list box so the item at position \e index in the list
+  becomes the bottom row of the list box.
+  \sa setTopItem()
+*/
+
+void QListBox::setBottomItem( int index )
+{
+    int i = index+1;
+    int y = 0;
+    while (i) y += cellHeight(--i);
+    y -= viewHeight();
+    setYOffset( y );
+}
+
 
 /*!
   Returns TRUE if drag-selection is enabled, otherwise FALSE.
@@ -1186,7 +1211,7 @@ void QListBox::setSmoothScrolling( bool enable )
 
 QListBoxItem *QListBox::item( int index ) const
 {
-    if (!checkIndex( "item", count(), index ) )
+    if (!checkIndex( "item", name(), count(), index ) )
 	return 0;
     return itemList->at( index );
 }
@@ -1266,8 +1291,7 @@ void QListBox::paintCell( QPainter *p, int row, int col )
     lbi->paint( p );
     if ( current == row && hasFocus() ) {
 	if ( style() == WindowsStyle ) {
-	    p->drawWinFocusRect( 1, 1, cellWidth(col)-2 , cellHeight(row)-2, 
-				 QApplication::winStyleHighlightColor() );
+	    p->drawWinFocusRect( 1, 1, cellWidth(col)-2 , cellHeight(row)-2 );
 	} else {
 	    if ( isSelected( row ) )
 		p->setPen( g.base() );
@@ -1291,11 +1315,11 @@ void QListBox::mousePressEvent( QMouseEvent *e )
 {
     int itemClicked = findItem( e->pos().y() );
     if ( itemClicked != -1 ) {
-	setCurrentItem( itemClicked );
+	ensureCurrentVisible( itemClicked );
 	toggleCurrentItem();
     } else if ( contentsRect().contains( e->pos() ) &&
 		lastRowVisible() >= (int) count() ) {
-	setCurrentItem( count()-1 );
+	ensureCurrentVisible( count()-1 );
 	toggleCurrentItem();
     }
 }
@@ -1342,15 +1366,19 @@ void QListBox::mouseMoveEvent( QMouseEvent *e )
 {
     if ( doDrag && (e->state() & (RightButton|LeftButton|MidButton)) != 0 ) {
 	int itemClicked = findItem( e->pos().y() );
-	if ( itemClicked != -1 ) {
+	if ( itemClicked >= 0 ) {
 	    if ( isTiming ) {
 		killTimer( itemList->timerId );
 		isTiming = FALSE;
 	    }
 	    if ( multiSelect ) {
+		bool s = currentItem() >= 0
+			 ? isSelected( currentItem() ) : TRUE;
 		int i = QMIN( itemClicked, currentItem() );
+		if ( i < 0 )
+		    i = 0;
 		while( i <= itemClicked || i <= currentItem() ) {
-		    setSelected( i, isSelected( currentItem() ) );
+		    setSelected( i, s );
 		    i++;
 		}
 	    }
@@ -1390,64 +1418,95 @@ void QListBox::keyPressEvent( QKeyEvent *e )
     if ( currentItem() < 0 )
 	setCurrentItem( topItem() );
 
-    int pageSize, delta;
+    int oldCurrent;
 
     switch ( e->key() ) {
     case Key_Up:
 	if ( currentItem() > 0 ) {
-	    setCurrentItem( currentItem() - 1 );
-	    if ( currentItem() < topItem()	)
-		setTopItem( currentItem() );
+	    ensureCurrentVisible( currentItem() - 1 );
 	    if ( e->state() & ShiftButton )
 		toggleCurrentItem();
 	}
+	e->accept();
 	break;
     case Key_Down:
 	if ( currentItem() < (int)count() - 1 ) {
-	    bool a = autoUpdate();
-	    bool u = FALSE;
-	    setAutoUpdate( FALSE );
-	    setCurrentItem( currentItem() + 1 );
-	    while ( currentItem() < (int)count() - 1
-		 && currentItem() >= lastRowVisible() )
-	    {
-		setTopItem( topItem() + currentItem() + 1 - lastRowVisible() );
-		u = TRUE;
-	    }
-	    setAutoUpdate( a );
+	    ensureCurrentVisible( currentItem()+1 );
 	    if ( e->state() & ShiftButton )
 		toggleCurrentItem();
-	    if ( u )
-		repaint();
 	}
+	e->accept();
+	break;
+    case Key_Left:
+    case Key_Right:
+	if ( bottomScrollBar() )
+	    QApplication::sendEvent( (QObject *)horizontalScrollBar(), e );
 	break;
     case Key_Next:
-	if ( style() == MotifStyle ) {
-	    delta = currentItem() - topItem();
-	    pageSize = QMAX( 1, lastRowVisible() - 1 - topItem() );
-	    setTopItem( QMIN( topItem() + pageSize, (int)count() - 1 ) );
-	    setCurrentItem( QMIN( topItem() + delta, (int)count() - 1 ) );
-	} else {
-	    pageSize = QMAX( 1, lastRowVisible() - 1 - topItem() );
-	    setTopItem( QMIN(currentItem(),(int)count()-pageSize+1) );
-	    setCurrentItem( QMIN(lastRowVisible(), (int)count()-1) );
+	e->accept();
+	if ( style() == MotifStyle) {
+	    if (lastRowVisible() == (int) count() - 1){
+		int o = yOffset();
+		setBottomItem( lastRowVisible() );
+		if ( currentItem() < lastRowVisible() && currentItem() == topItem()
+		     && yOffset() != o)
+		    setCurrentItem(currentItem() + 1);
+		break;
+	    }
+	    if (currentItem() != topItem() ){
+		setTopItem( currentItem() );
+		break;
+	    }
 	}
+	else {
+	    if (currentItem() != lastRowVisible() || lastRowVisible() == (int) count() - 1) {
+		ensureCurrentVisible(lastRowVisible());
+		break;
+	    }
+	}
+	oldCurrent = currentItem();
+	setYOffset(yOffset() + viewHeight() );
+	if ( style() == MotifStyle)
+	    ensureCurrentVisible( topItem() );
+	else
+	    ensureCurrentVisible(lastRowVisible());
+	if (oldCurrent == currentItem() && currentItem() + 1 <  (int) count() )
+	    ensureCurrentVisible( currentItem() + 1 );
 	break;
     case Key_Prior:
-	if ( style() == MotifStyle ) {
-	    delta = currentItem() - topItem();
-	    pageSize = QMAX( 1, lastRowVisible() - 1 - topItem() );
-	    setTopItem( QMAX( topItem() - pageSize, 0 ) );
-	    setCurrentItem( QMAX( topItem() + delta, 0 ) );
-	} else {
-	    pageSize = QMAX( 1, lastRowVisible() - topItem() );
-	    setTopItem( QMAX(0,currentItem()-pageSize+1) );
-	    setCurrentItem( topItem() );
+	e->accept();
+	if ( style() != MotifStyle) {
+	    if (currentItem() != topItem() || topItem() == 0){
+		ensureCurrentVisible(topItem());
+		break;
+	    }
 	}
+	else {
+	    if (topItem() == 0){
+		int o = yOffset();
+		setTopItem( topItem() );
+		if ( currentItem() > 0 && currentItem() == lastRowVisible() && yOffset() != o)
+		    setCurrentItem(currentItem()-1);
+		break;
+	    }
+	    if (currentItem() != lastRowVisible()) {
+		setBottomItem( currentItem() );
+		break;
+	    }
+	}
+	oldCurrent = currentItem();
+	setYOffset(yOffset() - viewHeight() );
+	if ( style() == MotifStyle)
+	    ensureCurrentVisible( lastRowVisible() );
+	else
+	    ensureCurrentVisible( topItem() );
+	if (oldCurrent == currentItem() && currentItem() > 0)
+	    ensureCurrentVisible( currentItem() -1);
 	break;
 
     case Key_Space:
 	toggleCurrentItem();
+	e->accept();
 	break;
 
     case Key_Return:
@@ -1458,6 +1517,7 @@ void QListBox::keyPressEvent( QKeyEvent *e )
 	    if ( !tmp.isEmpty() )
 		emit selected( tmp );
 	}
+	// do NOT accept here.  qdialog.
 	break;
     default:
 	break;
@@ -1468,7 +1528,7 @@ void QListBox::keyPressEvent( QKeyEvent *e )
 
 /*!
   Handles focus events.  Repaints the current item (if not set,
-  topItem() is made current).  
+  topItem() is made current).
   \sa keyPressEvent(), focusOutEvent()
 */
 
@@ -1481,7 +1541,7 @@ void QListBox::focusInEvent( QFocusEvent * )
 }
 
 
-/*! 
+/*!
   Handles focus out events. Repaints the current item, if set.
   \sa keyPressEvent(), focusOutEvent()
 */
@@ -1501,8 +1561,12 @@ void QListBox::focusOutEvent( QFocusEvent * )
 
 void QListBox::resizeEvent( QResizeEvent *e )
 {
+    bool u = autoUpdate();
+    setAutoUpdate( FALSE );
     QTableView::resizeEvent( e );
-    setCellWidth( QMAX(maxItemWidth(), viewWidth()) );
+    setCellWidth( (int)maxItemWidth() );
+    setCellWidth( QMAX( contentsRect().width(), (int)maxItemWidth()) );
+    setAutoUpdate( u );
 }
 
 
@@ -1517,21 +1581,27 @@ void QListBox::timerEvent( QTimerEvent * )
 	    int y = QMAX(currentItem(),lastRowVisible())+1;
 	    if ( y >= (int)count() )
 		y = count() - 1;
-	    int h = viewHeight();
-	    int i = y;
-	    while( i > 0 && h > 0 ) {
-		h -= cellHeight( i );
-		if ( h >= 0 )
-		    i--;
+	    if ( currentItem() >= 0 && multiSelect ) {
+		bool s = isSelected( currentItem() );
+		int i = currentItem();
+		while( i <= y ) {
+		    setSelected( i, s );
+		    i++;
+		}
 	    }
-	    if ( h <= 0 && i < y ) // make sure ALL of y is visible
-		i++;
-	    setTopItem( i );
-	    setCurrentItem( y );
+	    ensureCurrentVisible( y );
 	}
     } else {
-	if ( topItem() != 0 ) {
+	if ( topItem() > 0 ) {
 	    setTopItem( topItem() - 1 );
+	    if ( currentItem() > 0 && multiSelect ) {
+		bool s = isSelected( currentItem() );
+		int i = currentItem();
+		while( i >= topItem() ) {
+		    setSelected( i, s );
+		    i--;
+		}
+	    }
 	    setCurrentItem( topItem() );
 	}
     }
@@ -1636,7 +1706,7 @@ void QListBox::insert( const QListBoxItem *lbi, int index,
     ASSERT( (uint)index <= itemList->count() );
 #endif
     itemList->insert( index, lbi );
-    if ( current == index )
+    if ( current >= index )
 	current++;
     if ( updateCellWidth ) {
 	int w = lbi->width( this );
@@ -1705,16 +1775,28 @@ void QListBox::updateNumRows( bool updateWidth )
 
 
 /*!
-  Returns the width in pixels of the longest item. 
+  Returns the width in pixels of the widest item.
 */
 
-long QListBox::maxItemWidth()
+long QListBox::maxItemWidth() const
 {
     if ( !qlb_maxLenDict )
 	return 0;
     return (long) qlb_maxLenDict->find( (long)this );
 }
 
+/*!
+  For binary compatibility.
+*/
+long QListBox::maxItemWidth()
+{
+    // This is only here for binary compatibility
+    if ( !qlb_maxLenDict )
+	return 0;
+    return (long) qlb_maxLenDict->find( (long)this );
+    // This is only here for binary compatibility
+}
+
 
 /*!
   Updates the cached value used by maxItemWidth().
@@ -1744,7 +1826,7 @@ void QListBox::setMaxItemWidth( int len )
   single-selection mode if \a enable is FALSE.
 
   Single- and multi-selections modes work the same, except that the
-  highlighed() and selected() signals are emitted at different times.
+  highlighted() and selected() signals are emitted at different times.
 
   \sa isMultiSelection()
 */
@@ -1787,7 +1869,7 @@ void QListBox::toggleCurrentItem()
   unselects it if \a select is FALSE.  May also repaint the item.
 
   If the listbox is a single-selection listbox and and \a select is TRUE,
-  setCurrentItem will be called. 
+  setCurrentItem will be called.
 
   If the listbox is a single-selection listbox and and \a select is FALSE,
   clearSelection() will be called if \a index is the currently selected
@@ -1822,7 +1904,7 @@ void QListBox::setSelected( int index, bool select )
 
 
 /*!
-  Returns TRUE if item \a i is selected. Returns FALSE if it is not 
+  Returns TRUE if item \a i is selected. Returns FALSE if it is not
   selected or if there is an error.
 */
 
@@ -1859,10 +1941,14 @@ void QListBox::clearSelection()
     }
 }
 
+
+
+
+
 /*!  If \a lazy is FALSE, maybe emit the changed() signal.  If \a lazy
   is TRUE, note that it's to be sent out at some later time.
 */
- 
+
 void QListBox::emitChangedSignal( bool lazy ) {
     if ( !multiSelect )
 	return;
@@ -1872,3 +1958,53 @@ void QListBox::emitChangedSignal( bool lazy ) {
 
     changedListBox = lazy ? this : 0;
 }
+
+
+/*!  Make sure that all of currentItem() is on-screen */
+
+void QListBox::ensureCurrentVisible( int newCurrent )
+{
+    if ( newCurrent < 0 )
+	newCurrent = currentItem();
+    if ( newCurrent <= topItem() && newCurrent < lastRowVisible() )
+	 setTopItem( newCurrent );
+    else if ( newCurrent >= lastRowVisible() )
+	setBottomItem( newCurrent );
+    if ( newCurrent != currentItem() )
+	setCurrentItem( newCurrent );
+}
+
+/*!
+  Sets a \link QWidget::setFixedHeight() fixed height\endlink for the
+  widget, so that it shows the given number of lines of text for the
+  current font size.
+*/
+void QListBox::setFixedVisibleLines( int lines )
+{
+    int ls = fontMetrics().lineSpacing() + 1; // #### explain +1
+    // #### What about auto-scrollbars?
+    int sb = testTableFlags(Tbl_hScrollBar)
+		? horizontalScrollBar()->height() : 0;
+    setFixedHeight( frameWidth()*2 + ls*lines + sb );
+    return;
+}
+
+/*!
+  Returns a size mased on maxItemWidth() and any value set by
+  setFixedVisibleLines().
+*/
+QSize QListBox::sizeHint() const
+{
+    QSize sz = QTableView::sizeHint();
+
+    int w = (int)maxItemWidth() + 2*frameWidth();
+    if ( testTableFlags(Tbl_vScrollBar) )
+	w += verticalScrollBar()->width();
+    sz.setWidth(w);
+
+    // For when setFixedVisibleLines is used
+    int h = maximumSize().height();
+    if ( h<1000 ) sz.setHeight(h);
+
+    return sz;
+}
