@@ -1,13 +1,13 @@
 /****************************************************************************
-** $Id: qmenudata.cpp,v 2.20 1998/07/08 15:54:59 agulbra Exp $
+** $Id: qmenudata.cpp,v 2.20.2.1 1998/11/24 14:53:41 ettrich Exp $
 **
 ** Implementation of QMenuData class
 **
 ** Created : 941128
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/free-license.html.
@@ -184,8 +184,13 @@ int QMenuData::insertAny( const char *text, const QPixmap *pixmap,
 	index = mitems->count();
     if ( id < -1 )				// -2, -3 etc.
 	id = get_seq_id();
-    if ( popup && popup->parentMenu )		// popup already in use
-	return 0;
+    if ( popup && popup->parentMenu ){		// popup already in use
+#if defined(CHECK_STATE)
+	warning( "QMenuData::insertItem: "
+		 "popup already in use" );
+#endif
+  	return 0;
+    }
     register QMenuItem *mi = new QMenuItem;
     CHECK_PTR( mi );
     mi->ident = id == -1 ? index : id;
