@@ -1,13 +1,13 @@
 /****************************************************************************
-** $Id: qscrollview.cpp,v 2.50 1998/07/03 00:09:53 hanord Exp $
+** $Id: qscrollview.cpp,v 2.50.2.8 1998/11/20 12:24:03 warwick Exp $
 **
 ** Implementation of QScrollView class
 **
 ** Created : 950524
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of Qt Free Edition, version 1.45.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/free-license.html.
@@ -34,11 +34,10 @@
 
 const int sbDim = 16;
 
-struct ChildRec {
-    ChildRec(QWidget* c, int xx, int yy) :
+struct QSVChildRec {
+    QSVChildRec(QWidget* c, int xx, int yy) :
 	child(c),
-	x(xx), y(yy),
-	wantshown(TRUE)
+	x(xx), y(yy)
     {
     }
 
@@ -48,7 +47,7 @@ struct ChildRec {
     }
     void moveTo(QScrollView* sv, int xx, int yy)
     {
-	if ( x != xx || y != yy || wantshown ) {
+	if ( x != xx || y != yy ) {
 	    x = xx;
 	    y = yy;
 	    hideOrShow(sv);
@@ -56,24 +55,19 @@ struct ChildRec {
     }
     void hideOrShow(QScrollView* sv)
     {
-	if ( wantshown ) {
-	    if ( x-sv->contentsX() < -child->width()
-	      || x-sv->contentsX() > sv->viewport()->width()
-	      || y-sv->contentsY() < -child->height()
-	      || y-sv->contentsY() > sv->viewport()->height() )
-	    {
-		child->hide();
-	    } else {
-		child->move(x-sv->contentsX(), y-sv->contentsY());
-		child->show();
-	    }
+	if ( x-sv->contentsX() < -child->width()
+	  || x-sv->contentsX() > sv->viewport()->width()
+	  || y-sv->contentsY() < -child->height()
+	  || y-sv->contentsY() > sv->viewport()->height() )
+	{
+	    child->move(sv->viewport()->width()+10000,
+			sv->viewport()->height()+10000);
 	} else {
-	    child->hide();
+	    child->move(x-sv->contentsX(), y-sv->contentsY());
 	}
     }
     QWidget* child;
     int x, y;
-    bool wantshown;
 };
 
 struct QScrollViewData {
@@ -97,8 +91,8 @@ struct QScrollViewData {
 	deleteAll();
     }
 
-    ChildRec* rec(QWidget* w) { return childDict.find(w); }
-    ChildRec* ancestorRec(QWidget* w)
+    QSVChildRec* rec(QWidget* w) { return childDict.find(w); }
+    QSVChildRec* ancestorRec(QWidget* w)
     {
 	while (w->parentWidget() != &viewport) {
 	    w = w->parentWidget();
@@ -106,14 +100,14 @@ struct QScrollViewData {
 	}
 	return rec(w);
     }
-    ChildRec* addChildRec(QWidget* w, int x, int y )
+    QSVChildRec* addChildRec(QWidget* w, int x, int y )
     {
-	ChildRec *r = new ChildRec(w,x,y);
+	QSVChildRec *r = new QSVChildRec(w,x,y);
 	children.append(r);
 	childDict.insert(w, r);
 	return r;
     }
-    void deleteChildRec(ChildRec* r)
+    void deleteChildRec(QSVChildRec* r)
     {
 	childDict.remove(r->child);
 	children.removeRef(r);
@@ -121,33 +115,33 @@ struct QScrollViewData {
     }
     void hideOrShowAll(QScrollView* sv)
     {
-	for (ChildRec *r = children.first(); r; r=children.next()) {
+	for (QSVChildRec *r = children.first(); r; r=children.next()) {
 	    r->hideOrShow(sv);
 	}
     }
     void moveAllBy(int dx, int dy)
     {
-	for (ChildRec *r = children.first(); r; r=children.next()) {
+	for (QSVChildRec *r = children.first(); r; r=children.next()) {
 	    r->child->move(r->child->x()+dx,r->child->y()+dy);
 	}
     }
     void deleteAll()
     {
-	for (ChildRec *r = children.first(); r; r=children.next()) {
+	for (QSVChildRec *r = children.first(); r; r=children.next()) {
 	    delete r;
 	}
     }
     bool anyVisibleChildren()
     {
-	for (ChildRec *r = children.first(); r; r=children.next()) {
-	    if (r->wantshown) return TRUE;
+	for (QSVChildRec *r = children.first(); r; r=children.next()) {
+	    if (r->child->isVisible()) return TRUE;
 	}
 	return FALSE;
     }
     void autoResize(QScrollView* sv)
     {
 	if ( policy == QScrollView::AutoOne ) {
-	    ChildRec* r = children.first();
+	    QSVChildRec* r = children.first();
 	    sv->resizeContents(r->child->width(),r->child->height());
 	}
     }
@@ -156,8 +150,8 @@ struct QScrollViewData {
     QScrollBar	hbar;
     QScrollBar	vbar;
     QWidget	viewport;
-    QList<ChildRec>	children;
-    QPtrDict<ChildRec>	childDict;
+    QList<QSVChildRec>	children;
+    QPtrDict<QSVChildRec>	childDict;
     QWidget*	corner;
     int		vx, vy, vwidth, vheight; // for drawContents-style usage
     int		l_marg, r_marg, t_marg, b_marg;
@@ -178,6 +172,11 @@ QWidgets positioned on it that scroll around with the drawn content.  These
 subwidgets can also have positions outside the normal coordinate range
 (but they are still limited in size).
 
+Note that the scrolled area is the viewport() widget, not the QScrollView
+itself.  So, to turn mouse tracking on for example, use
+viewport()->setMouseTracking(TRUE).  The only part of the QScrollView that
+is visible is the "corner" and the frame.
+
 To provide content for the widget, inherit from QScrollView and
 override drawContentsOffset(), and use resizeContents() to set
 the size of the viewed area.  Use addChild(), moveChild(), and showChild()
@@ -185,9 +184,6 @@ to position widgets on the view.
 
 Note also the effect of resizePolicy().
 
-Currently the scrollbars fail beyond about
-1000000 pixels due to integer overflow.
-
 <img src=qscrollview-m.gif> <img src=qscrollview-w.gif>
 */
 
@@ -212,9 +208,9 @@ QScrollView::QScrollView( QWidget *parent, const char *name, WFlags f ) :
     d->viewport.installEventFilter( this );
 }
 
-/*!
-Destructs the QScrollView.  Any children added with addChild() will be destructed.
-*/
+/*! Destructs the QScrollView.  Any children added with addChild()
+  will be destructed. */
+
 QScrollView::~QScrollView()
 {
     // Be careful not to get all those useless events...
@@ -265,7 +261,7 @@ void QScrollView::updateScrollBars()
     bool needv;
     bool showh;
     bool showv;
-    
+
     if ( d->policy != AutoOne || d->anyVisibleChildren() ) {
 	// Do we definitely need the scrollbar?
 	needh = w-lmarg-rmarg < contentsWidth();
@@ -404,6 +400,7 @@ void QScrollView::resize( int w, int h )
 {
     // Need both this and resize event, due to deferred resize event.
     QWidget::resize( w, h );
+    d->hideOrShowAll(this);
     updateScrollBars();
 }
 
@@ -421,9 +418,11 @@ An override - ensures scrollbars are correct size upon resize.
 void QScrollView::resizeEvent( QResizeEvent* event )
 {
     QWidget::resizeEvent( event );
+    d->hideOrShowAll(this);
     updateScrollBars();
 }
 
+
 /*!
   Returns the currently set mode for the vertical scrollbar.
 
@@ -574,7 +573,7 @@ void QScrollView::removeChild(QWidget* child)
     if ( !d ) // In case we are destructing
 	return;
 
-    ChildRec *r = d->rec(child);
+    QSVChildRec *r = d->rec(child);
     if ( r ) d->deleteChildRec( r );
 }
 
@@ -592,11 +591,12 @@ void QScrollView::addChild(QWidget* child, int x, int y)
 {
     if ( child->parentWidget() == &d->viewport ) {
 	// May already be there
-	ChildRec *r = d->rec(child);
+	QSVChildRec *r = d->rec(child);
 	if (r) {
 	    r->moveTo(this,x,y);
-	    if ( d->policy > Manual )
+	    if ( d->policy > Manual ) {
 		d->autoResize(this); // #### better to just deal with this one widget!
+	    }
 	    return;
 	}
     }
@@ -610,11 +610,11 @@ void QScrollView::addChild(QWidget* child, int x, int y)
     if ( child->parentWidget() != &d->viewport ) {
 	child->recreate( &d->viewport, 0, QPoint(0,0), FALSE );
     }
-    child->hide();
     d->addChildRec(child,x,y)->hideOrShow(this);
 
-    if ( d->policy > Manual )
+    if ( d->policy > Manual ) {
 	d->autoResize(this); // #### better to just deal with this one widget!
+    }
 }
 
 /*!
@@ -645,24 +645,28 @@ int QScrollView::childY(QWidget* child)
 }
 
 /*!
-  Returns TRUE if \a child is visible.
-  Use this rather than QWidget::isVisible() for widgets added to the view.
+  \obsolete
+
+  Returns TRUE if \a child is visible.  This is equivalent
+  to child->isVisible().
 */
 bool QScrollView::childIsVisible(QWidget* child)
 {
-    return d->rec(child)->wantshown;
+    return child->isVisible();
 }
 
 /*!
-  Sets the visibility of \a child.
-  Use this rather than QWidget::show() or QWidget::hide()
-  for widgets added to the view.
+  \obsolete
+
+  Sets the visibility of \a child. Equivalent to
+  QWidget::show() or QWidget::hide().
 */
 void QScrollView::showChild(QWidget* child, bool y)
 {
-    ChildRec* r = d->rec(child);
-    r->wantshown = y;
-    r->hideOrShow(this);
+    if ( y )
+	child->show();
+    else
+	child->hide();
 }
 
 
@@ -698,7 +702,7 @@ bool QScrollView::eventFilter( QObject *obj, QEvent *e )
 	}
     } else {
 	// must be a child
-	ChildRec* r = d->rec((QWidget*)obj);
+	QSVChildRec* r = d->rec((QWidget*)obj);
 	if (!r) return FALSE; // spurious
 	switch ( e->type() ) {
 	  case Event_Resize:
@@ -1033,7 +1037,7 @@ void QScrollView::resizeContents( int w, int h )
   Note that the final coordinates you give to QPainter methods must be
   within the range supported by the underlying window systems - about
   +/- 32000 at most, often much less - +/- 4000 or so is all you should
-  rely expect.
+  really expect.
 
   For example:
   \code
@@ -1126,7 +1130,7 @@ void QScrollView::setMargins(int left, int top, int right, int bottom)
 	 right == d->r_marg &&
 	 bottom == d->b_marg )
 	return;
-	 
+	
     d->l_marg = left;
     d->t_marg = top;
     d->r_marg = right;
@@ -1180,51 +1184,38 @@ int QScrollView::bottomMargin() const
 */
 bool QScrollView::focusNextPrevChild( bool next )
 {
+    // first set things up for the scan
     QFocusData *f = focusData();
-
     QWidget *startingPoint = f->home();
-
-    if (!startingPoint)
-	return FALSE;
-
-    ChildRec *r = focusWidget() ? d->ancestorRec(focusWidget()) : 0;
-    if ( r && r->wantshown && !r->child->isVisible() )
-	return FALSE;
-
     QWidget *candidate = 0;
     QWidget *w = next ? f->next() : f->prev();
+    QSVChildRec *r;
 
-    while ( w && w != startingPoint ) {
-	while( w!=startingPoint ) {
-	    if ( w->testWFlags( WState_TabToFocus ) && !w->focusProxy() &&
-		 w->isEnabledToTLW() ) {
-		candidate = w;
-		break;
-	    }
-	    w = next ? f->next() : f->prev();
-	}
-
-	if ( !candidate )
-	    return FALSE;
-
-	r = d->ancestorRec(candidate);
-
-	if ( r ) {
-	    if ( candidate->isVisibleTo( r->child ) ) {
-		QPoint cp = candidate->mapToGlobal(QPoint(0,0));
-		QPoint cr = r->child->mapToGlobal(QPoint(0,0)) - cp;
-		ensureVisible( r->x+cr.x()+candidate->width()/2,
-			       r->y+cr.y()+candidate->height()/2,
-			       candidate->width()/2,
-			       candidate->height()/2 );
-		candidate->setFocus();
-		break;
-	    }
-	} else if ( candidate->isVisible() ) {
-	    candidate->setFocus();
-	    break;
-	}
+    // then scan for a possible focus widget candidate
+    while( !candidate && w != startingPoint ) {
+	if ( w != startingPoint && w->testWFlags( WState_TabToFocus ) &&
+	     w->isEnabledToTLW() &&!w->focusProxy() &&
+	     ( w->isVisibleToTLW() ) )
+	    candidate = w;
+	w = next ? f->next() : f->prev();
     }
 
+    // if we could not find one, maybe super or parentWidget() can?
+    if ( !candidate )
+	return QFrame::focusNextPrevChild( next );
+
+    // we've found one.
+    r = d->ancestorRec( candidate );
+    if ( r && ( r->child == candidate ||
+		candidate->isVisibleTo( r->child ) ) ) {
+	QPoint cp = r->child->mapToGlobal(QPoint(0,0));
+	QPoint cr = candidate->mapToGlobal(QPoint(0,0)) - cp;
+	ensureVisible( r->x+cr.x()+candidate->width()/2,
+		       r->y+cr.y()+candidate->height()/2,
+		       candidate->width()/2,
+		       candidate->height()/2 );
+    }
+
+    candidate->setFocus();
     return TRUE;
 }
